@@ -19,7 +19,7 @@
 %                               January 1993                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1996 E. I. Dupont de Nemours and Company                         %
+%  Copyright 1997 E. I. Dupont de Nemours and Company                         %
 %                                                                             %
 %  Permission to use, copy, modify, distribute, and sell this software and    %
 %  its documentation for any purpose is hereby granted without fee,           %
@@ -37,7 +37,7 @@
 %  and fitness, in no event shall E. I. Dupont de Nemours and Company be      %
 %  liable for any special, indirect or consequential damages or any           %
 %  damages whatsoever resulting from loss of use, data or profits, whether    %
-%  in an action of contract, negligence or other tortuous action, arising     %
+%  in an action of contract, negligence or other tortious action, arising     %
 %  out of or in connection with the use or performance of this software.      %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -61,10 +61,11 @@
 %    -font name          X11 font for displaying text
 %    -geometry geometry  perferred size or location of the image
 %    -gravity direction  which direction to gravitate towards
-%    -interlace type     NONE, LINE, or PLANE
+%    -interlace type     None, Line, Plane, or Partition
 %    -label name         assign a label to an image
 %    -matte              store matte channel if the image has one
 %    -monochrome         transform image to black and white
+%    -negate              apply color inversion to image
 %    -page geometry      size and location of the Postscript page
 %    -quality value      JPEG quality setting
 %    -scene value        image scene number
@@ -86,12 +87,6 @@
 #include "magick.h"
 #include "version.h"
 
-/*
-  Function prototypes.
-*/
-static void
-  Usage(void);
-
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -107,11 +102,14 @@ static void
 %
 %  The format of the Usage routine is:
 %
-%      Usage()
+%      Usage(client_name)
+%
+%    o client_name: a character string representing the name of the client
+%      program.
 %
 %
 */
-static void Usage(void)
+static void Usage(const char *client_name)
 {
   char
     **p;
@@ -133,10 +131,11 @@ static void Usage(void)
       "-font name          X11 font for displaying text",
       "-geometry geometry  perferred size or location of the image",
       "-gravity direction  which direction to gravitate towards",
-      "-interlace type     NONE, LINE, or PLANE",
+      "-interlace type     None, Line, Plane, or Partition",
       "-label name         assign a label to an image",
       "-matte              store matte channel if the image has one",
       "-monochrome         transform image to black and white",
+      "-negate              apply color inversion to image",
       "-page geometry      size and location of the Postscript page",
       "-quality value      JPEG quality setting",
       "-scene value        image scene number",
@@ -179,6 +178,7 @@ int main(int argc,char *argv[])
 #define NotInitialized  (unsigned int) (~0)
 
   char
+    *client_name,
     *displacement_geometry,
     *filename,
     *geometry,
@@ -201,13 +201,15 @@ int main(int argc,char *argv[])
     image_info;
 
   int
-    i,
+    gravity,
     x,
     y;
 
+  register int
+    i;
+
   unsigned int
     colorspace,
-    gravity,
     stereo,
     tile;
 
@@ -217,7 +219,7 @@ int main(int argc,char *argv[])
   ReadCommandlLine(argc,&argv);
   client_name=ClientName(*argv);
   if (argc < 4)
-    Usage();
+    Usage(client_name);
   /*
     Read image and convert to MIFF format.
   */
@@ -446,13 +448,13 @@ int main(int argc,char *argv[])
             }
           if (strncmp("dispose",option+1,5) == 0)
             {
-              image_info.dispose=0;
+              image_info.dispose=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
                     Error("Missing method on -dispose",(char *) NULL);
-                  image_info.dispose=atoi(argv[i]);
+                  image_info.dispose=PostscriptGeometry(argv[i]);
                 }
               break;
             }
@@ -534,7 +536,7 @@ int main(int argc,char *argv[])
         {
           if (strncmp("help",option+1,2) == 0)
             {
-              Usage();
+              Usage(client_name);
               break;
             }
           Error("Unrecognized option",option);
@@ -558,6 +560,8 @@ int main(int argc,char *argv[])
                     image_info.interlace=LineInterlace;
                   if (Latin1Compare("plane",option) == 0)
                     image_info.interlace=PlaneInterlace;
+                  if (Latin1Compare("partition",option) == 0)
+                    image_info.interlace=PartitionInterlace;
                   if (image_info.interlace == UndefinedInterlace)
                     Error("Invalid interlace type on -interlace",option);
                 }
@@ -566,15 +570,6 @@ int main(int argc,char *argv[])
           Error("Unrecognized option",option);
           break;
         }
-        case 'm':
-        {
-          if (strncmp("matte",option+1,5) == 0)
-            break;
-          if (strncmp("monochrome",option+1,2) == 0)
-            break;
-          Error("Unrecognized option",option);
-          break;
-        }
         case 'l':
         {
           if (strncmp("label",option+1,2) == 0)
@@ -590,6 +585,22 @@ int main(int argc,char *argv[])
           Error("Unrecognized option",option);
           break;
         }
+        case 'm':
+        {
+          if (strncmp("matte",option+1,5) == 0)
+            break;
+          if (strncmp("monochrome",option+1,2) == 0)
+            break;
+          Error("Unrecognized option",option);
+          break;
+        }
+        case 'n':
+        {
+          if (strncmp("negate",option+1,3) == 0)
+            break;
+          Error("Unrecognized option",option);
+          break;
+        }
         case 'p':
         {
           if (strncmp("page",option+1,3) == 0)
@@ -684,7 +695,7 @@ int main(int argc,char *argv[])
         }
         case '?':
         {
-          Usage();
+          Usage(client_name);
           break;
         }
         default:
@@ -695,7 +706,7 @@ int main(int argc,char *argv[])
       }
   }
   if ((image == (Image *) NULL) || (composite_image == (Image *) NULL))
-    Usage();
+    Usage(client_name);
   if (mask_image != (Image *) NULL)
     {
       CompositeImage(composite_image,AddMaskCompositeOp,mask_image,0,0);
