@@ -19,7 +19,7 @@
 %                                April 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1996 E. I. Dupont de Nemours and Company                         %
+%  Copyright 1997 E. I. Dupont de Nemours and Company                         %
 %                                                                             %
 %  Permission to use, copy, modify, distribute, and sell this software and    %
 %  its documentation for any purpose is hereby granted without fee,           %
@@ -37,7 +37,7 @@
 %  and fitness, in no event shall E. I. Dupont de Nemours and Company be      %
 %  liable for any special, indirect or consequential damages or any           %
 %  damages whatsoever resulting from loss of use, data or profits, whether    %
-%  in an action of contract, negligence or other tortuous action, arising     %
+%  in an action of contract, negligence or other tortious action, arising     %
 %  out of or in connection with the use or performance of this software.      %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -56,6 +56,7 @@
 %    -border geometry     surround image with a border of color
 %    -box color           color for annotation bounding box
 %    -charcoal factor     simulate a charcoal drawing
+%    -colorize value      colorize the image with the pen color
 %    -colors value        preferred number of colors in the image
 %    -colorspace type     GRAY, OHTA, RGB, XYZ, YCbCr, YIQ, YPbPr, or YUV
 %    -comment string      annotate image with comment
@@ -81,8 +82,9 @@
 %    -gamma value         level of gamma correction
 %    -geometry geometry   perferred size or location of the image
 %    -implode amount      implode image pixels about the center
-%    -interlace type      NONE, LINE, or PLANE
+%    -interlace type      None, Line, Plane, or Partition
 %    -label name          assign a label to an image
+%    -linewidth value     width of line in pixels
 %    -loop iterations     add Netscape loop extension to your GIF animation
 %    -map filename        transform image colors to match this set of colors
 %    -matte               store matte channel if the image has one
@@ -112,6 +114,7 @@
 %    -spread amount       displace image pixels by a random amount
 %    -swirl degrees       swirl image pixels about the center
 %    -texture filename    name of texture to tile onto the image background
+%    -threshold value     threshold the image
 %    -transparent color   make this color transparent within the image
 %    -treedepth value     depth of the color classification tree
 %    -undercolor geometry control undercolor removal and black generation
@@ -144,6 +147,7 @@
 %    GIF   CompuServe graphics interchange format; 8-bit color.
 %    GIF87 CompuServe graphics interchange format; 8-bit color (version 87a).
 %    GRADATION gradual passing from one shade to another.
+%    GRANITE granite texture
 %    GRAY  Raw gray bytes.
 %    HDF   Hierarchical Data Format.
 %    HISTOGRAM
@@ -155,7 +159,9 @@
 %    MIFF  Magick image file format.
 %    MONO  Bi-level bitmap in least-significant-byte (LSB) first order.
 %    MPEG  Motion Picture Experts Group file interchange format.
-%    MTV
+%    MTV   MTV Raytracing image format
+%    NETSCAPE
+%          Netscape 216 color cube.
 %    NULL  NULL image.
 %    PBM   Portable bitmap format (black and white).
 %    PCD   Photo CD.
@@ -163,6 +169,7 @@
 %    PDF   Portable Document Format.
 %    PGM   Portable graymap format (gray scale).
 %    PICT  Apple Macintosh QuickDraw/PICT file.
+%    PLASMA plasma fractal image.
 %    PNG   Portable Network Graphics.
 %    PNM   Portable anymap.
 %    PPM   Portable pixmap format (color).
@@ -191,7 +198,6 @@
 %    XPM   X Windows system pixmap file (color).
 %    XWD   X Windows system window dump file (color).
 %    YUV   CCIR 601 1:1:1 file.
-%    YUV3  CCIR 601 2:1:1 files.
 %
 %
 */
@@ -202,12 +208,6 @@
 #include "magick.h"
 #include "version.h"
 
-/*
-  Function prototypes.
-*/
-static void
-  Usage(void);
-
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -223,11 +223,14 @@ static void
 %
 %  The format of the Usage routine is:
 %
-%      Usage()
+%      Usage(client_name)
+%
+%    o client_name: a character string representing the name of the client
+%      program.
 %
 %
 */
-static void Usage(void)
+static void Usage(const char *client_name)
 {
   char
     **p;
@@ -251,6 +254,7 @@ static void Usage(void)
       "GIF   CompuServe graphics interchange format; 8-bit color.",
       "GIF87 CompuServe graphics interchange format; 8-bit color (version 87a).",
       "GRADATION gradual passing from one shade to another.",
+      "GRANITE granite texture.",
       "GRAY  Raw gray bytes.",
       "HDF   Hierarchical Data Format.",
       "HISTOGRAM",
@@ -262,7 +266,8 @@ static void Usage(void)
       "MIFF  Magick image file format.",
       "MONO  Bi-level bitmap in least-significant-byte (LSB) first order.",
       "MPEG  Motion Picture Experts Group file interchange format.",
-      "MTV",
+      "MTV   MTV Raytracing image format.",
+      "NETSCAPE  Netscape 216 color cube.",
       "NULL  NULL image.",
       "PBM   Portable bitmap format (black and white).",
       "PCD   Photo CD.",
@@ -270,6 +275,7 @@ static void Usage(void)
       "PDF   Portable Document Format.",
       "PGM   Portable graymap format (gray scale).",
       "PICT  Apple Macintosh QuickDraw/PICT file.",
+      "PLASMA plasma fractal image.",
       "PNG   Portable Network Graphics.",
       "PNM   Portable anymap.",
       "PPM   Portable pixmap format (color).",
@@ -297,7 +303,6 @@ static void Usage(void)
       "XPM   X Windows system pixmap file (color).",
       "XWD   X Windows system window dump file (color).",
       "YUV   CCIR 601 1:1:1 file.",
-      "YUV3  CCIR 601 2:1:1 files.",
       (char *) NULL,
     },
     *options[]=
@@ -308,6 +313,7 @@ static void Usage(void)
       "-border geometry     surround image with a border of color",
       "-box color           color for annotation bounding box",
       "-charcoal factor     simulate a charcoal drawing",
+      "-colorize value      colorize the image with the pen color",
       "-colors value        preferred number of colors in the image",
       "-colorspace type     GRAY, OHTA, RGB, XYZ, YCbCr, YIQ, YPbPr, or YUV",
       "-comment string      annotate image with comment",
@@ -333,7 +339,8 @@ static void Usage(void)
       "-gamma value         level of gamma correction",
       "-geometry geometry   perferred size or location of the image",
       "-implode amount      implode image pixels about the center",
-      "-interlace type      NONE, LINE, or PLANE",
+      "-interlace type      None, Line, Plane, or Partition",
+      "-linewidth value     width of line in pixels",
       "-label name          assign a label to an image",
       "-loop iterations     add Netscape loop extension to your GIF animation",
       "-map filename        transform image colors to match this set of colors",
@@ -364,6 +371,7 @@ static void Usage(void)
       "-spread amount       displace image pixels by a random amount",
       "-swirl degrees       swirl image pixels about the center",
       "-texture filename    name of texture to tile onto the image background",
+      "-threshold value     threshold the image",
       "-transparent color   make this color transparent within the image",
       "-treedepth value     depth of the color classification tree",
       "-undercolor geometry control undercolor removal and black generation",
@@ -414,12 +422,12 @@ int main(int argc,char *argv[])
 #define NotInitialized  (unsigned int) (~0)
 
   char
+    *client_name,
     *filename,
     *option;
 
   Image
     *image,
-    *map_image,
     *next_image;
 
   ImageInfo
@@ -433,6 +441,7 @@ int main(int argc,char *argv[])
 
   unsigned int
     average,
+    global_colormap,
     scene;
 
   /*
@@ -441,15 +450,16 @@ int main(int argc,char *argv[])
   ReadCommandlLine(argc,&argv);
   client_name=ClientName(*argv);
   if (argc < 3)
-    Usage();
+    Usage(client_name);
   /*
     Read image and convert to MIFF format.
   */
   average=False;
   filename=(char *) NULL;
   image=(Image *) NULL;
+  global_colormap=False;
   GetImageInfo(&image_info);
-  map_image=(Image *) NULL;
+  option=(char *) NULL;
   scene=0;
   /*
     Parse command-line arguments.
@@ -557,6 +567,16 @@ int main(int argc,char *argv[])
                 }
               break;
             }
+          if (strncmp("colorize",option+1,7) == 0)
+            {
+              if (*option == '-')
+                {
+                  i++;
+                  if ((i == argc) || !sscanf(argv[i],"%d",&x))
+                    Error("Missing value on -colorsize",(char *) NULL);
+                }
+              break;
+            }
           if (strncmp("colors",option+1,7) == 0)
             {
               if (*option == '-')
@@ -698,13 +718,13 @@ int main(int argc,char *argv[])
             }
           if (strncmp("dispose",option+1,5) == 0)
             {
-              image_info.dispose=0;
+              image_info.dispose=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
                     Error("Missing method on -dispose",(char *) NULL);
-                  image_info.dispose=atoi(argv[i]);
+                  image_info.dispose=PostscriptGeometry(argv[i]);
                 }
               break;
             }
@@ -807,7 +827,7 @@ int main(int argc,char *argv[])
         {
           if (strncmp("help",option+1,2) == 0)
             {
-              Usage();
+              Usage(client_name);
               break;
             }
           Error("Unrecognized option",option);
@@ -841,6 +861,8 @@ int main(int argc,char *argv[])
                     image_info.interlace=LineInterlace;
                   if (Latin1Compare("plane",option) == 0)
                     image_info.interlace=PlaneInterlace;
+                  if (Latin1Compare("partition",option) == 0)
+                    image_info.interlace=PartitionInterlace;
                   if (image_info.interlace == UndefinedInterlace)
                     Error("Invalid interlace type on -interlace",option);
                 }
@@ -860,15 +882,25 @@ int main(int argc,char *argv[])
                 }
               break;
             }
+          if (strncmp("linewidth",option+1,2) == 0)
+            {
+              if (*option == '-')
+                {
+                  i++;
+                  if ((i == argc) || !sscanf(argv[i],"%d",&x))
+                    Error("Missing size on -linewidth",(char *) NULL);
+                }
+              break;
+            }
           if (strncmp("loop",option+1,2) == 0)
             {
-              image_info.iterations=1;
+              image_info.iterations=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
                     Error("Missing iterations on -loop",(char *) NULL);
-                  image_info.iterations=atoi(argv[i]);
+                  image_info.iterations=PostscriptGeometry(argv[i]);
                 }
               break;
             }
@@ -879,15 +911,12 @@ int main(int argc,char *argv[])
         {
           if (strncmp("map",option+1,3) == 0)
             {
+              global_colormap=(*option == '+');
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     Error("Missing file name on -map",(char *) NULL);
-                  (void) strcpy(image_info.filename,argv[i]);
-                  map_image=ReadImage(&image_info);
-                  if (map_image == (Image *) NULL)
-                    Error("Unable to read map image",argv[i]);
                 }
               break;
             }
@@ -1062,7 +1091,7 @@ int main(int argc,char *argv[])
               if (*option == '-')
                 {
                   i++;
-                  if ((i == argc) || !sscanf(argv[i],"%f",(float *) &x))
+                  if ((i == argc) || !IsGeometry(argv[i]))
                     Error("Missing degrees on -rotate",(char *) NULL);
                 }
               break;
@@ -1190,6 +1219,16 @@ int main(int argc,char *argv[])
                 }
               break;
             }
+          if (strncmp("threshold",option+1,2) == 0)
+            {
+              if (*option == '-')
+                {
+                  i++;
+                  if ((i == argc) || !sscanf(argv[i],"%d",&x))
+                    Error("Missing value on -threshold",(char *) NULL);
+                }
+              break;
+            }
           if (strncmp("transparent",option+1,3) == 0)
             {
               if (*option == '-')
@@ -1243,7 +1282,7 @@ int main(int argc,char *argv[])
         }
         case '?':
         {
-          Usage();
+          Usage(client_name);
           break;
         }
         default:
@@ -1259,7 +1298,7 @@ int main(int argc,char *argv[])
     Write images.
   */
   if ((Extent(option) > 2) && ((*option == '-') || (*option == '+')))
-    MogrifyImage(&image_info,i,argv,&next_image);
+    MogrifyImages(&image_info,i,argv,&image);
   while (image->previous != (Image *) NULL)
     image=image->previous;
   if (average)
@@ -1277,6 +1316,8 @@ int main(int argc,char *argv[])
           image=averaged_image;
         }
     }
+  if (global_colormap)
+    MapImages(image,(Image *) NULL,image_info.dither);
   (void) strcpy(image_info.filename,argv[i]);
   SetImageInfo(&image_info,True);
   do
@@ -1286,8 +1327,6 @@ int main(int argc,char *argv[])
     */
     (void) strcpy(image->filename,argv[i]);
     image->scene=scene++;
-    if (map_image != (Image *) NULL)
-      MapImage(image,map_image,False);
     if (!image_info.adjoin)
       (void) WriteImage(&image_info,image);
     next_image=image->next;
@@ -1297,11 +1336,9 @@ int main(int argc,char *argv[])
   while (image->previous != (Image *) NULL)
     image=image->previous;
   if (image_info.adjoin)
-    {
-      (void) WriteImage(&image_info,image);
-      if (image_info.verbose)
-        DescribeImage(image,stderr,False);
-    }
+    (void) WriteImage(&image_info,image);
+  if (image_info.verbose)
+    DescribeImage(image,stderr,False);
   exit(0);
   return(False);
 }
