@@ -19,7 +19,7 @@
 %                              July 1992                                      %
 %                                                                             %
 %                                                                             %
-%  Copyright 1996 E. I. du Pont de Nemours and Company                        %
+%  Copyright 1997 E. I. du Pont de Nemours and Company                        %
 %                                                                             %
 %  Permission to use, copy, modify, distribute, and sell this software and    %
 %  its documentation for any purpose is hereby granted without fee,           %
@@ -37,7 +37,7 @@
 %  and fitness, in no event shall E. I. du Pont de Nemours and Company be     %
 %  liable for any special, indirect or consequential damages or any           %
 %  damages whatsoever resulting from loss of use, data or profits, whether    %
-%  in an action of contract, negligence or other tortuous action, arising     %
+%  in an action of contract, negligence or other tortious action, arising     %
 %  out of or in connection with the use or performance of this software.      %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -52,7 +52,7 @@
 %
 %  The import program command syntax is:
 %
-%  Usage: import [options ...] file
+%  Usage: import [options ...] [file]
 %
 %  Where options include:
 %    -border             include image borders in the output image
@@ -69,7 +69,7 @@
 %    -dither             apply Floyd/Steinberg error diffusion to image
 %    -frame              include window manager frame
 %    -geometry geometry  perferred size or location of the image
-%    -interlace type     NONE, LINE, or PLANE
+%    -interlace type     None, Line, Plane, or Partition
 %    -label name         assign a label to an image
 %    -monochrome         transform image to black and white
 %    -negate             apply color inversion to image
@@ -78,6 +78,7 @@
 %    -rotate degrees     apply Paeth rotation to the image
 %    -scene value        image scene number
 %    -screen             select image from root window
+%    -transparent color  make this color transparent within the image
 %    -treedepth value    depth of the color classification tree
 %    -verbose            print detailed information about the image
 %    -window id          select window with this id or name
@@ -101,10 +102,9 @@
 #include "version.h"
 
 /*
-  Function prototypes.
+  Define declarations.
 */
-static void
-  Usage(void);
+#define DefaultImportName  "magick.ps"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -121,11 +121,14 @@ static void
 %
 %  The format of the Usage routine is:
 %
-%      Usage()
+%      Usage(client_name)
+%
+%    o client_name: a character string representing the name of the client
+%      program.
 %
 %
 */
-static void Usage(void)
+static void Usage(const char *client_name)
 {
   char
     **p;
@@ -147,7 +150,7 @@ static void Usage(void)
       "-dither             apply Floyd/Steinberg error diffusion to image",
       "-frame              include window manager frame",
       "-geometry geometry  perferred size or location of the image",
-      "-interlace type     NONE, LINE, or PLANE",
+      "-interlace type     None, Line, Plane, or Partition",
       "-label name         assign a label to an image",
       "-monochrome         transform image to black and white",
       "-negate             apply color inversion to image",
@@ -156,6 +159,7 @@ static void Usage(void)
       "-rotate degrees     apply Paeth rotation to the image",
       "-scene value        image scene number",
       "-treedepth value    depth of the color classification tree",
+      "-transparent color   make this color transparent within the image",
       "-screen             select image from root window",
       "-verbose            print detailed information about the image",
       "-window id          select window with this id or name",
@@ -163,7 +167,7 @@ static void Usage(void)
     };
 
   (void) printf("Version: %s\n\n",Version);
-  (void) printf("Usage: %s [options ...] file\n",client_name);
+  (void) printf("Usage: %s [options ...] [file]\n",client_name);
   (void) printf("\nWhere options include:\n");
   for (p=options; *p != (char *) NULL; p++)
     (void) printf("  %s\n",*p);
@@ -199,6 +203,7 @@ static void Usage(void)
 int main(int argc,char *argv[])
 {
   char
+    *client_name,
     *filename,
     *option,
     *resource_value,
@@ -231,15 +236,10 @@ int main(int argc,char *argv[])
     resource_database;
 
   /*
-    Display usage profile if there are no command line arguments.
+    Connect to X server.
   */
   ReadCommandlLine(argc,&argv);
   client_name=ClientName(*argv);
-  if (argc < 2)
-    Usage();
-  /*
-    Connect to X server.
-  */
   image_info.server_name=(char *) NULL;
   for (i=1; i < argc; i++)
   {
@@ -261,7 +261,7 @@ int main(int argc,char *argv[])
         break;
       }
     if (strncmp("help",option+1,2) == 0)
-      Usage();
+      Usage(client_name);
   }
   display=XOpenDisplay(image_info.server_name);
   if (display == (Display *) NULL)
@@ -288,7 +288,7 @@ int main(int argc,char *argv[])
     XGetResourceClass(resource_database,client_name,"frame","False");
   frame=IsTrue(resource_value);
   resource_value=
-    XGetResourceClass(resource_database,client_name,"interlace","none");
+    XGetResourceClass(resource_database,client_name,"interlace","plane");
   image_info.interlace=UndefinedInterlace;
   if (Latin1Compare("none",resource_value) == 0)
     image_info.interlace=NoneInterlace;
@@ -296,6 +296,8 @@ int main(int argc,char *argv[])
     image_info.interlace=LineInterlace;
   if (Latin1Compare("plane",resource_value) == 0)
     image_info.interlace=PlaneInterlace;
+  if (Latin1Compare("partition",resource_value) == 0)
+    image_info.interlace=PartitionInterlace;
   if (image_info.interlace == UndefinedInterlace)
     Warning("Unrecognized interlace type",resource_value);
   image_info.page=XGetResourceClass(resource_database,client_name,
@@ -485,13 +487,13 @@ int main(int argc,char *argv[])
             }
           if (strncmp("dispose",option+1,5) == 0)
             {
-              image_info.dispose=0;
+              image_info.dispose=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
                     Error("Missing method on -dispose",(char *) NULL);
-                  image_info.dispose=atoi(argv[i]);
+                  image_info.dispose=PostscriptGeometry(argv[i]);
                 }
               break;
             }
@@ -528,7 +530,7 @@ int main(int argc,char *argv[])
         {
           if (strncmp("help",option+1,2) == 0)
             {
-              Usage();
+              Usage(client_name);
               break;
             }
           Error("Unrecognized option",option);
@@ -552,6 +554,8 @@ int main(int argc,char *argv[])
                     image_info.interlace=LineInterlace;
                   if (Latin1Compare("plane",option) == 0)
                     image_info.interlace=PlaneInterlace;
+                  if (Latin1Compare("partition",option) == 0)
+                    image_info.interlace=PartitionInterlace;
                   if (image_info.interlace == UndefinedInterlace)
                     Error("Invalid interlace type on -interlace",option);
                 }
@@ -637,7 +641,7 @@ int main(int argc,char *argv[])
               if (*option == '-')
                 {
                   i++;
-                  if ((i == argc) || !sscanf(argv[i],"%f",(float *) &x))
+                  if ((i == argc) || !IsGeometry(argv[i]))
                      Error("Missing degrees on -rotate",(char *) NULL);
                 }
               break;
@@ -665,6 +669,16 @@ int main(int argc,char *argv[])
         }
         case 't':
         {
+          if (strncmp("transparent",option+1,3) == 0)
+            {
+              if (*option == '-')
+                {
+                  i++;
+                  if (i == argc)
+                    Error("Missing color on -transparent",(char *) NULL);
+                }
+              break;
+            }
           if (strncmp("treedepth",option+1,3) == 0)
             {
               resource_info.tree_depth=0;
@@ -695,7 +709,7 @@ int main(int argc,char *argv[])
         }
         case '?':
         {
-          Usage();
+          Usage(client_name);
           break;
         }
         default:
@@ -706,7 +720,7 @@ int main(int argc,char *argv[])
       }
   }
   if (filename == (char *) NULL)
-    Error("Missing an image file name",(char *) NULL);
+    filename=DefaultImportName;
   /*
     Read image from X server.
   */
