@@ -19,7 +19,7 @@
 %                              July 1992                                      %
 %                                                                             %
 %                                                                             %
-%  Copyright 1996 E. I. du Pont de Nemours and Company                        %
+%  Copyright 1997 E. I. du Pont de Nemours and Company                        %
 %                                                                             %
 %  Permission to use, copy, modify, distribute, and sell this software and    %
 %  its documentation for any purpose is hereby granted without fee,           %
@@ -37,7 +37,7 @@
 %  and fitness, in no event shall E. I. du Pont de Nemours and Company be     %
 %  liable for any special, indirect or consequential damages or any           %
 %  damages whatsoever resulting from loss of use, data or profits, whether    %
-%  in an action of contract, negligence or other tortuous action, arising     %
+%  in an action of contract, negligence or other tortious action, arising     %
 %  out of or in connection with the use or performance of this software.      %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -54,7 +54,7 @@
 /*
   Define declarations.
 */
-#define MaxTreeDepth  8 
+#define MaxTreeDepth  8
 #define NodesInAList  2048
 
 /*
@@ -66,7 +66,7 @@ typedef struct _ColorsList
     red,
     green,
     blue;
- 
+
   unsigned long
     count;
 } ColorsList;
@@ -78,7 +78,7 @@ typedef struct _Node
 
   unsigned long
     number_unique;
-    
+
   ColorsList
     *list;
 
@@ -203,20 +203,20 @@ static void Histogram(const Node *node,FILE *file)
     {
       char
         name[MaxTextExtent];
- 
+
       ColorsList
         *p;
- 
+
       double
         distance_squared,
         min_distance;
- 
+
       int
         distance;
- 
+
       register int
         i;
- 
+
       register XColorlist
         *q;
 
@@ -326,7 +326,7 @@ static Node *InitializeNode(const unsigned int level)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function IsPseudoClass returns True if the image is PseudoClass or has 256
+%  Function IsPseudoClass returns True if the image is PseudoClass and has 256
 %  unique colors or less.  If the image is DirectClass and has less 256 colors
 %  or less, the image is demoted to PseudoClass.
 %
@@ -369,7 +369,8 @@ unsigned int IsPseudoClass(Image *image)
   unsigned int
     count;
 
-  if (image->class == PseudoClass)
+  assert(image != (Image *) NULL);
+  if ((image->class == PseudoClass) && (image->colors <= 256))
     return(True);
   if (image->matte)
     return(False);
@@ -387,7 +388,7 @@ unsigned int IsPseudoClass(Image *image)
       return(False);
     }
   p=image->pixels;
-  for (i=0; ((i < image->packets) && (cube.colors <= 256)); i++)
+  for (i=0; (i < image->packets) && (cube.colors <= 256); i++)
   {
     /*
       Start at the root and proceed level by level.
@@ -454,13 +455,18 @@ unsigned int IsPseudoClass(Image *image)
   while (cube.node_list != (Nodes *) NULL);
   if (cube.colors <= 256)
     {
+      QuantizeInfo
+        quantize_info;
+
       /*
         Demote DirectClass to PseudoClass.
       */
-      QuantizeImage(image,256,8,False,RGBColorspace);
+      GetQuantizeInfo(&quantize_info);
+      quantize_info.number_colors=256;
+      QuantizeImage(&quantize_info,image);
       SyncImage(image);
     }
-  return(image->class == PseudoClass);
+  return((image->class == PseudoClass) && (image->colors <= 256));
 }
 
 /*
@@ -494,7 +500,7 @@ unsigned int IsPseudoClass(Image *image)
 %
 %
 */
-void NumberColors(Image *image,FILE *file)
+Export void NumberColors(Image *image,FILE *file)
 {
 #define NumberColorsImageText  "  Computing image colors...  "
 
@@ -522,6 +528,7 @@ void NumberColors(Image *image,FILE *file)
   /*
     Initialize color description tree.
   */
+  assert(image != (Image *) NULL);
   image->total_colors=0;
   cube.node_list=(Nodes *) NULL;
   cube.colors=0;
