@@ -19,7 +19,7 @@
 %                              May  1993                                      %
 %                                                                             %
 %                                                                             %
-%  Copyright 1996 E. I. du Pont de Nemours and Company                        %
+%  Copyright 1997 E. I. du Pont de Nemours and Company                        %
 %                                                                             %
 %  Permission to use, copy, modify, distribute, and sell this software and    %
 %  its documentation for any purpose is hereby granted without fee,           %
@@ -272,13 +272,14 @@ void Ascii85Flush(FILE *file)
   register char
     *tuple;
 
+  assert(file != (FILE *) NULL);
   if (count > 0)
     {
       ascii85_buffer[count]=0;
       ascii85_buffer[count+1]=0;
       ascii85_buffer[count+2]=0;
       tuple=Ascii85Tuple(ascii85_buffer);
-      (void) fputs(*tuple == 'z' ? "!!!!" : tuple,file);
+      (void) fwrite(*tuple == 'z' ? "!!!!" : tuple,count+1,1,file);
     }
   (void) fputc('~',file);
   (void) fputc('>',file);
@@ -295,6 +296,7 @@ void Ascii85Encode(const unsigned int code,FILE *file)
   register unsigned char
     *p;
 
+  assert(file != (FILE *) NULL);
   ascii85_buffer[count]=code;
   count++;
   if (count < 4)
@@ -378,6 +380,8 @@ unsigned int BMPDecodeImage(unsigned char *compressed_pixels,
     byte,
     count;
 
+  assert(compressed_pixels != (unsigned char *) NULL);
+  assert(pixels != (unsigned char *) NULL);
   byte=0;
   x=0;
   p=compressed_pixels;
@@ -520,6 +524,8 @@ unsigned int BMPEncodeImage(unsigned char *pixels,
   /*
     Runlength encode pixels.
   */
+  assert(pixels != (unsigned char *) NULL);
+  assert(compressed_pixels != (unsigned char *) NULL);
   p=pixels;
   q=compressed_pixels;
   i=0;
@@ -622,6 +628,7 @@ unsigned int GIFDecodeImage(Image *image)
   /*
     Allocate decoder tables.
   */
+  assert(image != (Image *) NULL);
   packet=(unsigned char *) malloc(256*sizeof(unsigned char));
   prefix=(short *) malloc(MaxStackSize*sizeof(short));
   suffix=(unsigned char *) malloc(MaxStackSize*sizeof(unsigned char));
@@ -870,6 +877,7 @@ unsigned int GIFEncodeImage(Image *image,const unsigned int data_size)
   /*
     Allocate encoder tables.
   */
+  assert(image != (Image *) NULL);
   packet=(unsigned char *) malloc(256*sizeof(unsigned char));
   hash_code=(short *) malloc(MaxHashTable*sizeof(short));
   hash_prefix=(short *) malloc(MaxHashTable*sizeof(short));
@@ -899,7 +907,7 @@ unsigned int GIFEncodeImage(Image *image,const unsigned int data_size)
   waiting_code=p->index;
   for (i=0; i < image->packets; i++)
   {
-    for (j=(i == 0) ? 1 : 0; j <= (int) p->length; j++)
+    for (j=(i == 0) ? 1 : 0; j <= ((int) p->length); j++)
     {
       /*
         Probe hash table.
@@ -1030,6 +1038,8 @@ unsigned int HuffmanDecodeImage(Image *image)
   mask>>=1;  \
   if (bit)  \
     runlength=0;  \
+  if (feof(image->file))  \
+    break;  \
 }
 
   HuffmanTable
@@ -1038,6 +1048,7 @@ unsigned int HuffmanDecodeImage(Image *image)
     **mw_hash;
 
   int
+    bail,
     code,
     color,
     count,
@@ -1071,6 +1082,7 @@ unsigned int HuffmanDecodeImage(Image *image)
   /*
     Allocate buffers.
   */
+  assert(image != (Image *) NULL);
   mb_hash=(HuffmanTable **) malloc(HashSize*sizeof(HuffmanTable *));
   mw_hash=(HuffmanTable **) malloc(HashSize*sizeof(HuffmanTable *));
   scanline=(unsigned char *) malloc(image->columns*sizeof(unsigned char));
@@ -1098,13 +1110,18 @@ unsigned int HuffmanDecodeImage(Image *image)
   /*
     Uncompress 1D Huffman to runlength encoded pixels.
   */
+  byte=0;
   mask=0;
   null_lines=0;
+  runlength=0;
+  while (runlength < 11)
+   InputBit(bit);
+  do { InputBit(bit); } while (bit == 0);
   packets=image->packets;
   image->packets=0;
   q=image->pixels;
   q->length=MaxRunlength;
-  for (y=0; ((y < image->rows) && (null_lines < 3)); y++)
+  for (y=0; ((y < image->rows) && (null_lines < 3)); )
   {
     /*
       Initialize scanline to white.
@@ -1120,8 +1137,17 @@ unsigned int HuffmanDecodeImage(Image *image)
     count=0;
     length=0;
     runlength=0;
-    for (x=0; x < image->columns; )
+    x=0;
+    for ( ; ; )
     {
+      if (x >= image->columns)
+        {
+          while (runlength < 11)
+           InputBit(bit);
+          do { InputBit(bit); } while (bit == 0);
+          break;
+        }
+      bail=False;
       do
       {
         if (runlength < 11)
@@ -1130,13 +1156,26 @@ unsigned int HuffmanDecodeImage(Image *image)
           {
             InputBit(bit);
             if (bit)
-              length=13;
+              {
+                null_lines++;
+                if (x != 0)
+                  null_lines=0;
+                bail=True;
+                break;
+              }
           }
         code=(code << 1)+bit;
         length++;
-      } while ((code <= 0) && (length < 14));
-      if (length > 13)
+      } while (code <= 0);
+      if (bail)
         break;
+      if (length > 13)
+        {
+          while (runlength < 11)
+           InputBit(bit);
+          do { InputBit(bit); } while (bit == 0);
+          break;
+        }
       if (color)
         {
           if (length < 4)
@@ -1151,9 +1190,8 @@ unsigned int HuffmanDecodeImage(Image *image)
         }
       if (!entry)
         continue;
-      else
-        if ((entry->length != length) || (entry->code != code))
-          continue;
+      if ((entry->length != length) || (entry->code != code))
+        continue;
       switch (entry->id)
       {
         case TWId:
@@ -1176,10 +1214,6 @@ unsigned int HuffmanDecodeImage(Image *image)
         }
         case MWId:
         case MBId:
-        {
-          count+=entry->count;
-          break;
-        }
         case EXId:
         {
           count+=entry->count;
@@ -1191,20 +1225,6 @@ unsigned int HuffmanDecodeImage(Image *image)
       code=0;
       length=0;
     }
-    null_lines++;
-    if (x != 0)
-      {
-        /*
-          Skip to end of scanline.
-        */
-        while (runlength < 11)
-          InputBit(bit);
-        do
-        {
-          InputBit(bit);
-        } while (bit == 0);
-        null_lines=0;
-      }
     /*
       Transfer scanline to image pixels.
     */
@@ -1235,9 +1255,9 @@ unsigned int HuffmanDecodeImage(Image *image)
           q->length=0;
         }
     }
-    ProgressMonitor(LoadImageText,y,image->rows);
+    ProgressMonitor(LoadImageText,y++,image->rows);
   }
-  image->rows=y-1;
+  image->rows=y;
   SyncImage(image);
   image->pixels=(RunlengthPacket *)
     realloc((char *) image->pixels,image->packets*sizeof(RunlengthPacket));
@@ -1265,18 +1285,19 @@ unsigned int HuffmanDecodeImage(Image *image)
 %
 %  The format of the HuffmanEncodeImage routine is:
 %
-%      status=HuffmanEncodeImage(image)
+%      status=HuffmanEncodeImage(image_info,image)
 %
 %  A description of each parameter follows:
 %
 %    o status:  Function HuffmanEncodeImage returns True if all the pixels are
 %      compressed without error, otherwise False.
 %
+%    o image_info: Specifies a pointer to an ImageInfo structure.
+%
 %    o image: The address of a structure of type Image.
 %
-%
 */
-unsigned int HuffmanEncodeImage(Image *image)
+unsigned int HuffmanEncodeImage(const ImageInfo *image_info,Image *image)
 {
 #define HuffmanOutputCode(entry)  \
 {  \
@@ -1295,7 +1316,10 @@ unsigned int HuffmanEncodeImage(Image *image)
   bit>>=1;  \
   if ((bit & 0xff) == 0)   \
     {  \
-      (void) fputc((char) byte,image->file);  \
+      if (strcmp(image_info->magick,"FAX") == 0) \
+        (void) fputc((char) byte,image->file);  \
+      else \
+        Ascii85Encode((unsigned int) byte,image->file); \
       byte=0;  \
       bit=0x80;  \
     }  \
@@ -1329,18 +1353,33 @@ unsigned int HuffmanEncodeImage(Image *image)
     *scanline;
 
   unsigned int
-    mask;
+    mask,
+    width;
 
   /*
     Allocate scanline buffer.
   */
-  scanline=(unsigned char *)
-    malloc((Max(image->columns,1728)+1)*sizeof(unsigned char));
+  assert(image != (Image *) NULL);
+  width=image->columns;
+  if (strcmp(image_info->magick,"FAX") == 0)
+    width=Max(image->columns,1728);
+  scanline=(unsigned char *) malloc((width+1)*sizeof(unsigned char));
   if (scanline == (unsigned char *) NULL)
     {
       Warning("Unable to allocate memory",(char *) NULL);
       return(False);
     }
+  if (strcmp(image_info->magick,"FAX") != 0)
+    Ascii85Initialize();
+  else
+    {
+      /*
+        End of line.
+      */
+      for (k=0; k < 11; k++)
+        OutputBit(0);
+      OutputBit(1);
+    }
   /*
     Compress runlength encoded to 1D Huffman pixels.
   */
@@ -1349,7 +1388,7 @@ unsigned int HuffmanEncodeImage(Image *image)
     polarity=(Intensity(image->colormap[0]) >
       Intensity(image->colormap[1]) ? 0 : 1);
   q=scanline;
-  for (i=0; i < (Max(image->columns,1728)+1); i++)
+  for (i=0; i < width; i++)
     *q++=polarity;
   byte=0;
   bit=0x80;
@@ -1368,7 +1407,7 @@ unsigned int HuffmanEncodeImage(Image *image)
         Huffman encode scanline.
       */
       q=scanline;
-      for (n=Max(image->columns,1728); n > 0; )
+      for (n=width; n > 0; )
       {
         /*
           Output white run.
@@ -1381,10 +1420,12 @@ unsigned int HuffmanEncodeImage(Image *image)
         if (runlength >= 64)
           {
             entry=MWTable+((runlength/64)-1);
+            if (runlength >= 1792)
+              entry=EXTable+(Min(runlength,2560)-1792)/64;
             runlength-=entry->count;
             HuffmanOutputCode(entry);
           }
-        entry=TWTable+runlength;
+        entry=TWTable+Min(runlength,63);
         HuffmanOutputCode(entry);
         if (n != 0)
           {
@@ -1399,10 +1440,12 @@ unsigned int HuffmanEncodeImage(Image *image)
             if (runlength >= 64)
               {
                 entry=MBTable+((runlength/64)-1);
+                if (runlength >= 1792)
+                  entry=EXTable+(Min(runlength,2560)-1792)/64;
                 runlength-=entry->count;
                 HuffmanOutputCode(entry);
               }
-            entry=TBTable+runlength;
+            entry=TBTable+Min(runlength,63);
             HuffmanOutputCode(entry);
           }
       }
@@ -1432,7 +1475,12 @@ unsigned int HuffmanEncodeImage(Image *image)
     Flush bits.
   */
   if (bit != 0x80)
-    (void) fputc((char) byte,image->file);
+    if (strcmp(image_info->magick,"FAX") == 0)
+      (void) fputc((char) byte,image->file);
+    else
+      Ascii85Encode((unsigned int) byte,image->file);
+  if (strcmp(image_info->magick,"FAX") != 0)
+    Ascii85Flush(image->file);
   free((char *) scanline);
   return(True);
 }
@@ -1485,6 +1533,8 @@ unsigned int PackbitsEncodeImage(FILE *file,unsigned char *pixels,
   /*
     Compress pixels with Packbits encoding.
   */
+  assert(file != (FILE *) NULL);
+  assert(pixels != (unsigned char *) NULL);
   packbits=(unsigned char *) malloc(128*sizeof(unsigned char));
   if (packbits == (unsigned char *) NULL)
     {
@@ -1610,7 +1660,8 @@ unsigned int PackbitsEncodeImage(FILE *file,unsigned char *pixels,
 %
 %
 */
-unsigned int PCDDecodeImage(Image *image,unsigned char *luma,unsigned char *chroma1,unsigned char *chroma2)
+unsigned int PCDDecodeImage(Image *image,unsigned char *luma,
+  unsigned char *chroma1,unsigned char *chroma2)
 {
 #define IsSync  ((accumulator & 0xffffff00) == 0xfffffe00)
 #define PCDDecodeImageText  "  PCD decode image...  "
@@ -1629,6 +1680,8 @@ unsigned int PCDDecodeImage(Image *image,unsigned char *luma,unsigned char *chro
     bits+=8; \
     p++; \
   } \
+  if (feof(image->file)) \
+    break; \
 }
 
   typedef struct PCDTable
@@ -1680,6 +1733,10 @@ unsigned int PCDDecodeImage(Image *image,unsigned char *luma,unsigned char *chro
   /*
     Initialize Huffman tables.
   */
+  assert(image != (Image *) NULL);
+  assert(luma != (unsigned char *) NULL);
+  assert(chroma1 != (unsigned char *) NULL);
+  assert(chroma2 != (unsigned char *) NULL);
   buffer=(unsigned char *) malloc(0x800*sizeof(unsigned char));
   if (buffer == (unsigned char *) NULL)
     {
@@ -1732,8 +1789,8 @@ unsigned int PCDDecodeImage(Image *image,unsigned char *luma,unsigned char *chro
   */
   accumulator=0;
   bits=32;
-  PCDGetBits(16);
-  PCDGetBits(16);
+  do { PCDGetBits(16) } while (0);
+  do { PCDGetBits(16) } while (0);
   while ((accumulator & 0x00fff000) != 0x00fff000)
     PCDGetBits(8);
   while (!IsSync)
@@ -1883,6 +1940,9 @@ unsigned int PICTEncodeImage(Image *image,unsigned char *scanline,unsigned char
   /*
     Pack scanline.
   */
+  assert(image != (Image *) NULL);
+  assert(scanline != (unsigned char *) NULL);
+  assert(packbits != (unsigned char *) NULL);
   bytes_per_line=image->columns;
   if (image->class == DirectClass)
     bytes_per_line*=3;
@@ -2026,15 +2086,18 @@ unsigned int RunlengthDecodeImage(Image *image)
   unsigned short
     value;
 
+  assert(image != (Image *) NULL);
   if (image->packed_pixels == (unsigned char *) NULL)
     return(True);
   /*
     Allocate pixels.
   */
-  if (image->pixels != (RunlengthPacket *) NULL)
-    free((char *) image->pixels);
-  image->pixels=(RunlengthPacket *)
-    malloc((unsigned int) image->packets*sizeof(RunlengthPacket));
+  if (image->pixels == (RunlengthPacket *) NULL)
+    image->pixels=(RunlengthPacket *)
+      malloc((unsigned int) image->packets*sizeof(RunlengthPacket));
+  else
+    image->pixels=(RunlengthPacket *) realloc((char *) image->pixels,
+      image->packets*sizeof(RunlengthPacket));
   if (image->pixels == (RunlengthPacket *) NULL)
     {
       Warning("Unable to unpack pixels","Memory allocation failed");
@@ -2054,7 +2117,9 @@ unsigned int RunlengthDecodeImage(Image *image)
           ReadQuantum(q->red,p);
           ReadQuantum(q->green,p);
           ReadQuantum(q->blue,p);
-          q->index=(unsigned short) (image->matte ? (*p++) : 0);
+          q->index=0;
+          if (image->matte)
+            ReadQuantum(q->index,p);
           q->length=(*p++);
           count+=(q->length+1);
           q++;
@@ -2067,7 +2132,9 @@ unsigned int RunlengthDecodeImage(Image *image)
           ReadQuantum(q->red,p);
           ReadQuantum(q->green,p);
           ReadQuantum(q->blue,p);
-          q->index=(unsigned short) (image->matte ? (*p++) : 0);
+          q->index=0;
+          if (image->matte)
+            ReadQuantum(q->index,p);
           q->length=0;
           count++;
           q++;
@@ -2168,12 +2235,12 @@ unsigned int RunlengthDecodeImage(Image *image)
 %
 %  The format of the RunlengthEncodeImage routine is:
 %
-%      status=RunlengthEncodeImage(image)
+%      count=RunlengthEncodeImage(image)
 %
 %  A description of each parameter follows:
 %
-%    o status: Function RunlengthEncodeImage return True if the image is
-%      packed.  False is returned if an error occurs.
+%    o status: Function RunlengthEncodeImage return the number of bytes the
+%      image consumes.  Zero is returned if an error occurs.
 %
 %    o image: The address of a structure of type Image.
 %
@@ -2200,12 +2267,11 @@ unsigned int RunlengthEncodeImage(Image *image)
   unsigned short
     value;
 
-  if (image == (Image *) NULL)
-    return(False);
+  assert(image != (Image *) NULL);
   if (image->pixels == (RunlengthPacket *) NULL)
     {
       Warning("Unable to pack pixels","no image pixels");
-      return(False);
+      return(0);
     }
   if (image->compression == RunlengthEncodedCompression)
     {
@@ -2224,7 +2290,7 @@ unsigned int RunlengthEncodeImage(Image *image)
               Uncompress image to allow in-place compression.
             */
             if (!UncompressImage(image))
-              return(False);
+              return(0);
             break;
           }
         p++;
@@ -2306,24 +2372,26 @@ unsigned int RunlengthEncodeImage(Image *image)
     {
       image->packet_size=3*(image->depth >> 3);
       if (image->matte)
-        image->packet_size++;
+        image->packet_size+=image->depth >> 3;
     }
   if (image->compression == RunlengthEncodedCompression)
     image->packet_size++;
   /*
     Allocate packed pixel memory.
   */
-  if (image->packed_pixels != (unsigned char *) NULL)
-    free((char *) image->packed_pixels);
   packets=image->packets;
   if (image->compression != RunlengthEncodedCompression)
     packets=image->columns*image->rows;
-  image->packed_pixels=(unsigned char *)
-    malloc((unsigned int) packets*image->packet_size*sizeof(unsigned char));
+  if (image->packed_pixels == (unsigned char *) NULL)
+    image->packed_pixels=(unsigned char *)
+      malloc((unsigned int) packets*image->packet_size*sizeof(unsigned char));
+  else
+    image->packed_pixels=(unsigned char *) realloc((char *)
+      image->packed_pixels,packets*image->packet_size*sizeof(unsigned char));
   if (image->packed_pixels == (unsigned char *) NULL)
     {
       Warning("Unable to pack pixels","Memory allocation failed");
-      return(False);
+      return(0);
     }
   /*
     Packs the runlength-encoded pixel packets into the minimum number of bytes.
@@ -2340,7 +2408,7 @@ unsigned int RunlengthEncodeImage(Image *image)
           WriteQuantum(p->green,q);
           WriteQuantum(p->blue,q);
           if (image->matte)
-            *q++=(unsigned char) p->index;
+            WriteQuantum(p->index,q);
           *q++=p->length;
           count+=(p->length+1);
           p++;
@@ -2356,7 +2424,7 @@ unsigned int RunlengthEncodeImage(Image *image)
             WriteQuantum(p->green,q);
             WriteQuantum(p->blue,q);
             if (image->matte)
-              *q++=(unsigned char) p->index;
+              WriteQuantum(p->index,q);
           }
           count+=(p->length+1);
           p++;
@@ -2421,22 +2489,21 @@ unsigned int RunlengthEncodeImage(Image *image)
               ProgressMonitor(SaveImageText,i,image->packets);
           }
         }
-  image->packets=(unsigned int) packets;
   /*
     Guarentee the correct number of pixel packets.
   */
   if (count < (image->columns*image->rows))
     {
       Warning("insufficient image data in",image->filename);
-      return(False);
+      return(0);
     }
   else
     if (count > (image->columns*image->rows))
       {
         Warning("too much image data in",image->filename);
-        return(False);
+        return(0);
       }
-  return(True);
+  return((unsigned int) packets);
 }
 
 /*
@@ -2493,6 +2560,8 @@ unsigned int SUNDecodeImage(unsigned char *compressed_pixels,
   unsigned char
     byte;
 
+  assert(compressed_pixels != (unsigned char *) NULL);
+  assert(pixels != (unsigned char *) NULL);
   p=compressed_pixels;
   q=pixels;
   while ((q-pixels) <= (number_columns*number_rows))
