@@ -7,7 +7,7 @@ extern unsigned int
   BMPEncodeImage(unsigned char *,unsigned char *,const unsigned int,
     const unsigned int),
   HuffmanDecodeImage(Image *),
-  HuffmanEncodeImage(Image *),
+  HuffmanEncodeImage(const ImageInfo *image_info,Image *),
   GIFDecodeImage(Image *),
   GIFEncodeImage(Image *,const unsigned int),
   LZWEncodeImage(FILE *,unsigned char *,const unsigned int),
