@@ -39,13 +39,15 @@ struct dirent
 /*
   Macintosh utilities routines.
 */
-   
 extern long
   telldir(DIR *);
 
 extern DIR
   *opendir(char *);
- 
+
+extern int
+  systemMAC(const char *);
+
 extern struct dirent
   *readdir(DIR *);
  
