@@ -17,7 +17,7 @@
 %                             John Cristy                                     %
 %                              July 1992                                      %
 %                                                                             %
-%  Copyright 1996 E. I. du Pont de Nemours and Company                        %
+%  Copyright 1997 E. I. du Pont de Nemours and Company                        %
 %                                                                             %
 %  Permission to use, copy, modify, distribute, and sell this software and    %
 %  its documentation for any purpose is hereby granted without fee,           %
@@ -35,7 +35,7 @@
 %  and fitness, in no event shall E. I. du Pont de Nemours and Company be     %
 %  liable for any special, indirect or consequential damages or any           %
 %  damages whatsoever resulting from loss of use, data or profits, whether    %
-%  in an action of contract, negligence or other tortuous action, arising     %
+%  in an action of contract, negligence or other tortious action, arising     %
 %  out of or in connection with the use or performance of this software.      %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -322,33 +322,18 @@ static void
 %
 %  The format of the Assignment routine is:
 %
-%      Assignment(image,number_colors,dither,colorspace)
+%      Assignment(quantize_info,image)
 %
 %  A description of each parameter follows.
 %
+%    o quantize_info: Specifies a pointer to an QuantizeInfo structure.
+%
 %    o image: Specifies a pointer to an Image structure;  returned from
 %      ReadImage.
 %
-%    o number_colors: This integer value indicates the maximum number of
-%      colors in the quantized image or colormap.  Here we use this value
-%      to determine the depth of the color description tree.
-%
-%    o dither: Set this integer value to something other than zero to
-%      dither the quantized image.  The basic strategy of dithering is to
-%      trade intensity resolution for spatial resolution by averaging the
-%      intensities of several neighboring pixels.  Images which suffer
-%      from severe contouring when quantized can be improved with the
-%      technique of dithering.  Severe contouring generally occurs when
-%      quantizing to very few colors, or to a poorly-chosen colormap.
-%      Note, dithering is a computationally expensive process and will
-%      increase processing time significantly.
-%
-%    o colorspace: An unsigned integer value that indicates the colorspace.
-%
 %
 */
-static void Assignment(Image *image,const unsigned int number_colors,
-  unsigned int dither,const ColorspaceType colorspace)
+static void Assignment(QuantizeInfo *quantize_info,Image *image)
 {
 #define AssignImageText  "  Assigning image colors...  "
 
@@ -370,10 +355,12 @@ static void Assignment(Image *image,const unsigned int number_colors,
   /*
     Allocate image colormap.
   */
-  if (image->colormap != (ColorPacket *) NULL)
-    free((char *) image->colormap);
-  image->colormap=(ColorPacket *)
-    malloc(cube.colors*sizeof(ColorPacket));
+  if (image->colormap == (ColorPacket *) NULL)
+    image->colormap=(ColorPacket *)
+      malloc(cube.colors*sizeof(ColorPacket));
+  else
+    image->colormap=(ColorPacket *)
+      realloc((char *) image->colormap,cube.colors*sizeof(ColorPacket));
   if (image->colormap == (ColorPacket *) NULL)
     {
       Warning("Unable to quantize image","Memory allocation failed");
@@ -385,7 +372,8 @@ static void Assignment(Image *image,const unsigned int number_colors,
   cube.colormap=image->colormap;
   cube.colors=0;
   DefineColormap(cube.root);
-  if ((number_colors == 2)  && (colorspace == GRAYColorspace))
+  if ((quantize_info->number_colors == 2) &&
+      (quantize_info->colorspace == GRAYColorspace))
     {
       unsigned int
         polarity;
@@ -401,7 +389,7 @@ static void Assignment(Image *image,const unsigned int number_colors,
       image->colormap[!polarity].green=MaxRGB;
       image->colormap[!polarity].blue=MaxRGB;
     }
-  if (colorspace != TransparentColorspace)
+  if (quantize_info->colorspace != TransparentColorspace)
     {
       image->matte=False;
       image->class=PseudoClass;
@@ -410,10 +398,10 @@ static void Assignment(Image *image,const unsigned int number_colors,
   /*
     Create a reduced color image.
   */
-  if (dither)
-    dither=!DitherImage(image);
+  if (quantize_info->dither)
+    quantize_info->dither=!DitherImage(image);
   p=image->pixels;
-  if (!dither)
+  if (!quantize_info->dither)
     for (i=0; i < image->packets; i++)
     {
       /*
@@ -1004,6 +992,38 @@ static unsigned int DitherImage(Image *image)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   X G e t Q u a n t i z e I n f o                                           %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  Function XGetMontageInfo initializes the MontageInfo structure.
+%
+%  The format of the GetMontageInfo routine is:
+%
+%      XGetMontageInfo(quantize_info)
+%
+%  A description of each parameter follows:
+%
+%    o quantize_info: Specifies a pointer to a MontageInfo structure.
+%
+%
+*/
+Export void GetQuantizeInfo(QuantizeInfo *quantize_info)
+{
+  assert(quantize_info != (QuantizeInfo *) NULL);
+  quantize_info->number_colors=MaxRGB+1;
+  quantize_info->tree_depth=8;
+  quantize_info->dither=False;
+  quantize_info->colorspace=RGBColorspace;
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %  I n i t i a l i z e C u b e                                                %
 %                                                                             %
 %                                                                             %
@@ -1370,20 +1390,13 @@ static void Reduction(const unsigned int number_colors)
 %
 %  A description of each parameter follows:
 %
-%    o image: Specifies a pointer to a set of Image structures.
+%    o image: Specifies a pointer to an Image structure.
 %
 %    o map_image: Specifies a pointer to a Image structure.  Reduce
 %      image to a set of colors represented by this image.
 %
 %    o dither: Set this integer value to something other than zero to
-%      dither the quantized image.  The basic strategy of dithering is to
-%      trade intensity resolution for spatial resolution by averaging the
-%      intensities of several neighboring pixels.  Images which suffer
-%      from severe contouring when quantized can be improved with the
-%      technique of dithering.  Severe contouring generally occurs when
-%      quantizing to very few colors, or to a poorly-chosen colormap.
-%      Note, dithering is a computationally expensive process and will
-%      increase processing time significantly.
+%      dither the quantized image.
 %
 %
 */
@@ -1392,12 +1405,116 @@ void MapImage(Image *image,Image *map_image,const unsigned int dither)
   Nodes
     *nodes;
 
+  QuantizeInfo
+    quantize_info;
+
+  assert(image != (Image *) NULL);
+  if (map_image == (Image *) NULL)
+    return;
   /*
     Classify image colors from the reference image.
   */
   InitializeCube(8);
   Classification(map_image);
-  Assignment(image,(unsigned int) cube.colors,dither,RGBColorspace);
+  quantize_info.number_colors=cube.colors;
+  quantize_info.dither=dither;
+  quantize_info.colorspace=image->matte ? TransparentColorspace : RGBColorspace;
+  Assignment(&quantize_info,image);
+  /*
+    Release color cube tree storage.
+  */
+  do
+  {
+    nodes=cube.node_queue->next;
+    free((char *) cube.node_queue);
+    cube.node_queue=nodes;
+  } while (cube.node_queue != (Nodes *) NULL);
+  cube.squares-=MaxRGB;
+  free((char *) cube.squares);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   M a p I m a g e s                                                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MapImages replaces the colors of a sequence of images with the closest
+%  color from a reference image.
+%
+%  The format of the MapImage routine is:
+%
+%      MapImages(images,map_image,dither)
+%
+%  A description of each parameter follows:
+%
+%    o image: Specifies a pointer to a set of Image structures.
+%
+%    o map_image: Specifies a pointer to a Image structure.  Reduce
+%      image to a set of colors represented by this image.
+%
+%    o dither: Set this integer value to something other than zero to
+%      dither the quantized image.
+%
+%
+*/
+Export void MapImages(Image *images,Image *map_image,const unsigned int dither)
+{
+  Image
+    *image;
+
+  Nodes
+    *nodes;
+
+  QuantizeInfo
+    quantize_info;
+
+  assert(images != (Image *) NULL);
+  if (images->next == (Image *) NULL)
+    {
+      /*
+        Handle a single image with MapImage.
+      */
+      MapImage(images,map_image,dither);
+      return;
+    }
+  GetQuantizeInfo(&quantize_info);
+  quantize_info.dither=dither;
+  if (map_image == (Image *) NULL)
+    {
+      /*
+        Create a global colormap for an image sequence.
+      */
+      if (images->class == PseudoClass)
+        quantize_info.number_colors=images->colors;
+      for (image=images; image != (Image *) NULL; image=image->next)
+      {
+        if (image->matte)
+          quantize_info.colorspace=TransparentColorspace;
+        if (image->class == PseudoClass)
+          if (image->colors > quantize_info.number_colors)
+            quantize_info.number_colors=image->colors;
+      }
+      QuantizeImages(&quantize_info,images);
+      return;
+    }
+  /*
+    Classify image colors from the reference image.
+  */
+  InitializeCube(8);
+  Classification(map_image);
+  quantize_info.number_colors=cube.colors;
+  for (image=images; image != (Image *) NULL; image=image->next)
+  {
+    quantize_info.colorspace=
+      image->matte ? TransparentColorspace : RGBColorspace;
+    Assignment(&quantize_info,image);
+  }
   /*
     Release color cube tree storage.
   */
@@ -1574,6 +1691,7 @@ void QuantizationError(Image *image)
   /*
     Initialize measurement.
   */
+  assert(image != (Image *) NULL);
   image->mean_error_per_pixel=0;
   image->normalized_mean_error=0.0;
   image->normalized_maximum_error=0.0;
@@ -1639,53 +1757,16 @@ void QuantizationError(Image *image)
 %
 %  The format of the QuantizeImage routine is:
 %
-%      colors=QuantizeImage(image,number_colors,tree_depth,dither,color_space)
+%      QuantizeImage(quantize_info,image)
 %
-%  A description of each parameter follows.
+%  A description of each parameter follows:
 %
-%    o colors: The QuantizeImage function returns this integer
-%      value.  It is the actual number of colors allocated in the
-%      colormap.  Note, the actual number of colors allocated may be less
-%      than the number of colors requested, but never more.
-%
-%    o image: Specifies a pointer to an Image structure;  returned from
-%      ReadImage.
-%
-%    o number_colors: This integer value indicates the maximum number of
-%      colors in the quantized image or colormap.  The actual number of
-%      colors allocated to the colormap may be less than this value, but
-%      never more.
-%
-%    o tree_depth: Normally, this integer value is zero or one.  A zero or
-%      one tells Quantize to choose a optimal tree depth of
-%      Log4(number_colors).  A tree of this depth generally allows the best
-%      representation of the reference image with the least amount of memory
-%      and the fastest computational speed.  In some cases, such as an image
-%      with low color dispersion (a few number of colors), a value other than
-%      Log4(number_colors) is required.  To expand the color tree completely,
-%      use a value of 8.
-%
-%    o dither: Set this integer value to something other than zero to
-%      dither the quantized image.  The basic strategy of dithering is to
-%      trade intensity resolution for spatial resolution by averaging the
-%      intensities of several neighboring pixels.  Images which suffer
-%      from severe contouring when quantized can be improved with the
-%      technique of dithering.  Severe contouring generally occurs when
-%      quantizing to very few colors, or to a poorly-chosen colormap.
-%      Note, dithering is a computationally expensive process and will
-%      increase processing time significantly.
-%
-%    o colorspace: An unsigned integer value that indicates the colorspace.
-%      Empirical evidence suggests that distances in YUV or YIQ correspond to
-%      perceptual color differences more closely than do distances in RGB
-%      space.  The image is then returned to RGB colorspace after color
-%      reduction.
+%    o quantize_info: Specifies a pointer to an QuantizeInfo structure.
 %
+%    o image: Specifies a pointer to a Image structure.
 %
 */
-void QuantizeImage(Image *image,unsigned int number_colors,
-  const unsigned int tree_depth,unsigned int dither,
-  const ColorspaceType colorspace)
+Export void QuantizeImage(QuantizeInfo *quantize_info,Image *image)
 {
   int
     depth;
@@ -1693,18 +1774,20 @@ void QuantizeImage(Image *image,unsigned int number_colors,
   Nodes
     *nodes;
 
-  if ((number_colors == 2) && (colorspace == GRAYColorspace) && dither)
+  assert(image != (Image *) NULL);
+  if ((quantize_info->number_colors == 2) &&
+      (quantize_info->colorspace == GRAYColorspace) && quantize_info->dither)
     {
       OrderedDitherImage(image);
       return;
     }
-  if (number_colors == 0)
-    number_colors=1;
-  if (number_colors > MaxColormapSize)
-    number_colors=MaxColormapSize;
+  if (quantize_info->number_colors == 0)
+    quantize_info->number_colors=1;
+  if (quantize_info->number_colors > MaxColormapSize)
+    quantize_info->number_colors=MaxColormapSize;
   if (image->packets == (image->columns*image->rows))
     CompressImage(image);
-  depth=tree_depth;
+  depth=quantize_info->tree_depth;
   if (depth == 0)
     {
       unsigned int
@@ -1713,10 +1796,10 @@ void QuantizeImage(Image *image,unsigned int number_colors,
       /*
         Depth of color classification tree is: Log4(colormap size)+2.
       */
-      colors=number_colors;
+      colors=quantize_info->number_colors;
       for (depth=1; colors != 0; depth++)
         colors>>=2;
-      if (dither)
+      if (quantize_info->dither)
         depth--;
       if (image->class == PseudoClass)
         depth+=2;
@@ -1725,16 +1808,16 @@ void QuantizeImage(Image *image,unsigned int number_colors,
     Reduce the number of colors in the continuous tone image.
   */
   InitializeCube(depth);
-  if (colorspace != RGBColorspace)
-    RGBTransformImage(image,colorspace);
+  if (quantize_info->colorspace != RGBColorspace)
+    RGBTransformImage(image,quantize_info->colorspace);
   Classification(image);
-  if ((cube.colors >> 1) < number_colors)
-    dither=False;
-  if (number_colors < cube.colors)
-    Reduction(number_colors);
-  Assignment(image,number_colors,dither,colorspace);
-  if (colorspace != RGBColorspace)
-    TransformRGBImage(image,colorspace);
+  if ((cube.colors >> 1) < quantize_info->number_colors)
+    quantize_info->dither=False;
+  if (quantize_info->number_colors < cube.colors)
+    Reduction(quantize_info->number_colors);
+  Assignment(quantize_info,image);
+  if (quantize_info->colorspace != RGBColorspace)
+    TransformRGBImage(image,quantize_info->colorspace);
   /*
     Release color cube tree storage.
   */
@@ -1766,65 +1849,53 @@ void QuantizeImage(Image *image,unsigned int number_colors,
 %
 %  The format of the QuantizeImages routine is:
 %
-%      QuantizeImages(images,number_images,number_colors,tree_depth,dither)
+%      QuantizeImages(quantize_info,number_images,images)
 %
 %  A description of each parameter follows:
 %
-%    o images: Specifies a pointer to a set of Image structures.
+%    o quantize_info: Specifies a pointer to an QuantizeInfo structure.
 %
 %    o number_images:  Specifies an unsigned integer representing the number
 %      images in the image set.
 %
-%    o number_colors: This integer value indicates the maximum number of
-%      colors in the quantized image or colormap.  The actual number of colors
-%      allocated to the colormap may be less than this value, but never more.
-%
-%    o tree_depth: Normally, this integer value is zero or one.  A zero or
-%      one tells QUANTIZE to choose a optimal tree depth.  An optimal depth
-%      generally allows the best representation of the reference image with the
-%      fastest computational speed and the least amount of memory.  However,
-%      the default depth is inappropriate for some images.  To assure the best
-%      representation, try values between 2 and 8 for this parameter.
-%
-%    o dither: Set this integer value to something other than zero to
-%      dither the quantized image.  The basic strategy of dithering is to
-%      trade intensity resolution for spatial resolution by averaging the
-%      intensities of several neighboring pixels.  Images which suffer
-%      from severe contouring when quantized can be improved with the
-%      technique of dithering.  Severe contouring generally occurs when
-%      quantizing to very few colors, or to a poorly-chosen colormap.
-%      Note, dithering is a computationally expensive process and will
-%      increase processing time significantly.
-%
-%    o colorspace: An unsigned integer value that indicates the colorspace.
-%      Empirical evidence suggests that distances in YUV or YIQ correspond to
-%      perceptual color differences more closely than do distances in RGB
-%      space.  The image is then returned to RGB colorspace after color
-%      reduction.
+%    o images: Specifies a pointer to a list of Image structures.
 %
 %
 */
-void QuantizeImages(Image **images,const unsigned int number_images,
-  unsigned int number_colors,const unsigned int tree_depth,unsigned int dither,
-  const ColorspaceType colorspace)
+Export void QuantizeImages(QuantizeInfo *quantize_info,Image *images)
 {
   int
     depth;
 
+  MonitorHandler
+    handler;
+
   Image
     *image;
 
   Nodes
     *nodes;
 
-  register unsigned int
+  register int
     i;
 
-  if (number_colors == 0)
-    number_colors=1;
-  if (number_colors > MaxColormapSize)
-    number_colors=MaxColormapSize;
-  depth=tree_depth;
+  unsigned int
+    number_images;
+
+  assert(images != (Image *) NULL);
+  if (images->next == (Image *) NULL)
+    {
+      /*
+        Handle a single image with QuantizeImage.
+      */
+      QuantizeImage(quantize_info,images);
+      return;
+    }
+  if (quantize_info->number_colors == 0)
+    quantize_info->number_colors=1;
+  if (quantize_info->number_colors > MaxColormapSize)
+    quantize_info->number_colors=MaxColormapSize;
+  depth=quantize_info->tree_depth;
   if (depth == 0)
     {
       int
@@ -1836,14 +1907,14 @@ void QuantizeImages(Image **images,const unsigned int number_images,
       /*
         Depth of color classification tree is: Log4(colormap size)+2.
       */
-      colors=number_colors;
+      colors=quantize_info->number_colors;
       for (depth=1; colors != 0; depth++)
         colors>>=2;
-      if (dither)
+      if (quantize_info->dither)
         depth--;
       pseudo_class=True;
-      for (i=0; i < number_images; i++)
-        pseudo_class|=(images[i]->class == PseudoClass);
+      for (image=images; image != (Image *) NULL; image=image->next)
+        pseudo_class|=(image->class == PseudoClass);
       if (pseudo_class)
         depth+=2;
     }
@@ -1851,27 +1922,39 @@ void QuantizeImages(Image **images,const unsigned int number_images,
     Reduce the number of colors in the continuous tone image sequence.
   */
   InitializeCube(depth);
-  for (i=0; i < number_images; i++)
-    for (image=images[i]; image != (Image *) NULL; image=image->next)
-    {
-      if (image->packets == (image->columns*image->rows))
-        CompressImage(image);
-      if (colorspace != RGBColorspace)
-        RGBTransformImage(image,colorspace);
-    }
-  for (i=0; i < number_images; i+=Max((number_images+16) >> 5,1))
-    for (image=images[i]; image != (Image *) NULL; image=image->next)
-      Classification(image);
-  if ((cube.colors >> 1) < number_colors)
-    dither=False;
-  Reduction(number_colors);
-  for (i=0; i < number_images; i++)
-    for (image=images[i]; image != (Image *) NULL; image=image->next)
-    {
-      Assignment(image,number_colors,dither,colorspace);
-      if (colorspace != RGBColorspace)
-        TransformRGBImage(image,colorspace);
-    }
+  image=images;
+  for (i=0; image != (Image *) NULL; i++)
+  {
+    if (image->packets == (image->columns*image->rows))
+      CompressImage(image);
+    if (quantize_info->colorspace != RGBColorspace)
+      RGBTransformImage(image,quantize_info->colorspace);
+    image=image->next;
+  }
+  number_images=i;
+  image=images;
+  for (i=0; image != (Image *) NULL; i++)
+  {
+    handler=SetMonitorHandler((MonitorHandler) NULL);
+    Classification(image);
+    image=image->next;
+    (void) SetMonitorHandler(handler);
+    ProgressMonitor(ClassifyImageText,i,number_images);
+  }
+  if ((cube.colors >> 1) < quantize_info->number_colors)
+    quantize_info->dither=False;
+  Reduction(quantize_info->number_colors);
+  image=images;
+  for (i=0; image != (Image *) NULL; i++)
+  {
+    handler=SetMonitorHandler((MonitorHandler) NULL);
+    Assignment(quantize_info,image);
+    if (quantize_info->colorspace != RGBColorspace)
+      TransformRGBImage(image,quantize_info->colorspace);
+    image=image->next;
+    (void) SetMonitorHandler(handler);
+    ProgressMonitor(AssignImageText,i,number_images);
+  }
   /*
     Release color cube tree storage.
   */
