@@ -19,7 +19,7 @@
 %                            December 1992                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1996 E. I. du Pont de Nemours and Company                        %
+%  Copyright 1997 E. I. du Pont de Nemours and Company                        %
 %                                                                             %
 %  Permission to use, copy, modify, distribute, and sell this software and    %
 %  its documentation for any purpose is hereby granted without fee,           %
@@ -54,6 +54,7 @@
 %    -border geometry     surround image with a border of color
 %    -box color           color for annotation bounding box
 %    -charcoal factor     simulate a charcoal drawing
+%    -colorize value      colorize the image with the pen color
 %    -colors value        preferred number of colors in the image
 %    -colorspace type     GRAY, OHTA, RGB, XYZ, YCbCr, YIQ, YPbPr, or YUV
 %    -comment string      annotate image with comment
@@ -61,6 +62,7 @@
 %    -contrast            enhance or reduce the image contrast
 %    -crop geometry       preferred size and location of the cropped image
 %    -cycle amount        cycle the image colormap
+%    -delay value         display the next image after pausing
 %    -density geometry    vertical and horizontal density of the image
 %    -despeckle           reduce the speckles within an image",
 %    -display server      obtain image or font from this X server
@@ -79,8 +81,10 @@
 %    -gamma value         level of gamma correction
 %    -geometry geometry   perferred size or location of the image
 %    -implode amount      implode image pixels about the center
-%    -interlace type      NONE, LINE, or PLANE
+%    -interlace type      None, Line, Plane, or Partition
 %    -label name          assign a label to an image
+%    -linewidth value     width of line in pixels
+%    -loop iterations     add Netscape loop extension to your GIF animation
 %    -map filename        transform image colors to match this set of colors
 %    -modulate value      vary the brightness, saturation, and hue
 %    -monochrome          transform image to black and white
@@ -108,6 +112,7 @@
 %    -spread amount       displace image pixels by a random amount
 %    -swirl degrees       swirl image pixels about the center
 %    -texture filename    name of texture to tile onto the image background
+%    -threshold value     threshold the image
 %    -transparent color   make this color transparent within the image
 %    -treedepth value     depth of the color classification tree
 %    -undercolor geometry control undercolor removal and black generation
@@ -131,12 +136,6 @@
 #include "magick.h"
 #include "version.h"
 
-/*
-  Function prototypes.
-*/
-static void
-  Usage(void);
-
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -152,11 +151,14 @@ static void
 %
 %  The format of the Usage routine is:
 %
-%      Usage()
+%      Usage(client_name)
+%
+%    o client_name: a character string representing the name of the client
+%      program.
 %
 %
 */
-static void Usage(void)
+static void Usage(const char *client_name)
 {
   char
     **p;
@@ -168,6 +170,7 @@ static void Usage(void)
       "-border geometry     surround image with a border of color",
       "-box color           color for annotation bounding box",
       "-charcoal factor     simulate a charcoal drawing",
+      "-colorize value      colorize the image with the pen color",
       "-colors value        preferred number of colors in the image",
       "-colorspace type     GRAY, OHTA, RGB, XYZ, YCbCr, YIQ, YPbPr, or YUV",
       "-comment string      annotate image with comment",
@@ -175,6 +178,7 @@ static void Usage(void)
       "-contrast            enhance or reduce the image contrast",
       "-crop geometry       preferred size and location of the cropped image",
       "-cycle amount        cycle the image colormap",
+      "-delay value         display the next image after pausing",
       "-density geometry    vertical and horizontal density of the image",
       "-despeckle           reduce the speckles within an image",
       "-display server      obtain image or font from this X server",
@@ -193,8 +197,10 @@ static void Usage(void)
       "-gamma value         level of gamma correction",
       "-geometry geometry   perferred size or location of the image",
       "-implode amount      implode image pixels about the center",
-      "-interlace type      NONE, LINE, or PLANE",
+      "-interlace type      None, Line, Plane, or Partition",
       "-label name          assign a label to an image",
+      "-linewidth value     width of line in pixels",
+      "-loop iterations     add Netscape loop extension to your GIF animation",
       "-map filename        transform image colors to match this set of colors",
       "-modulate value      vary the brightness, saturation, and hue",
       "-monochrome          transform image to black and white",
@@ -222,6 +228,7 @@ static void Usage(void)
       "-spread amount       displace image pixels by a random amount",
       "-swirl degrees       swirl image pixels about the center",
       "-texture filename    name of texture to tile onto the image background",
+      "-threshold value     threshold the image",
       "-transparent color   make this color transparent within the image",
       "-treedepth value     depth of the color classification tree",
       "-undercolor geometry control undercolor removal and black generation",
@@ -267,29 +274,27 @@ static void Usage(void)
 int main(int argc,char **argv)
 {
   char
+    *client_name,
     filename[MaxTextExtent],
     *format,
     *option;
 
-  Image
-    **images,
-    *map_image;
-
   ImageInfo
     image_info;
 
+  Image
+    *image;
+
   int
-    i,
     x;
 
+  register int
+    i;
+
   unsigned int
-    colorspace,
-    image_number,
-    maximum_images,
-    number_colors,
+    global_colormap,
     scene,
-    status,
-    tree_depth;
+    status;
 
   /*
     Display usage profile if there are no command line arguments.
@@ -297,25 +302,18 @@ int main(int argc,char **argv)
   ReadCommandlLine(argc,&argv);
   client_name=ClientName(*argv);
   if (argc < 2)
-    Usage();
+    Usage(client_name);
   /*
     Set defaults.
   */
-  colorspace=RGBColorspace;
   format=(char *) NULL;
+  global_colormap=False;
+  image=(Image *) NULL;
   GetImageInfo(&image_info);
-  map_image=(Image *) NULL;
-  number_colors=0;
   scene=0;
-  tree_depth=0;
-  maximum_images=MaxTextExtent;
-  images=(Image **) malloc(maximum_images*sizeof(Image *));
-  if (images == (Image **) NULL)
-    Error("Unable to mogrify images","Memory allocation failed");
   /*
     Parse command line.
   */
-  image_number=0;
   ExpandFilenames(&argc,&argv);
   for (i=1; i < argc; i++)
   {
@@ -380,34 +378,40 @@ int main(int argc,char **argv)
                 }
               break;
             }
+          if (strncmp("colorize",option+1,7) == 0)
+            {
+              if (*option == '-')
+                {
+                  i++;
+                  if ((i == argc) || !sscanf(argv[i],"%d",&x))
+                    Error("Missing value on -colorsize",(char *) NULL);
+                }
+              break;
+            }
           if (strncmp("colors",option+1,7) == 0)
             {
-              number_colors=0;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
                     Error("Missing colors on -colors",(char *) NULL);
-                  number_colors=atoi(argv[i]);
                 }
               break;
             }
           if (strncmp("colorspace",option+1,7) == 0)
             {
-              colorspace=RGBColorspace;
               if (*option == '-')
                 {
+                  ColorspaceType
+                    colorspace;
+
                   i++;
                   if (i == argc)
                     Error("Missing type on -colorspace",(char *) NULL);
                   option=argv[i];
                   colorspace=UndefinedColorspace;
                   if (Latin1Compare("gray",option) == 0)
-                    {
-                      colorspace=GRAYColorspace;
-                      number_colors=256;
-                      tree_depth=8;
-                    }
+                    colorspace=GRAYColorspace;
                   if (Latin1Compare("ohta",option) == 0)
                     colorspace=OHTAColorspace;
                   if (Latin1Compare("rgb",option) == 0)
@@ -485,6 +489,18 @@ int main(int argc,char **argv)
         }
         case 'd':
         {
+          if (strncmp("delay",option+1,3) == 0)
+            {
+              image_info.delay=(char *) NULL;
+              if (*option == '-')
+                {
+                  i++;
+                  if ((i == argc) || !sscanf(argv[i],"%d",&x))
+                    Error("Missing delay on -delay",(char *) NULL);
+                  image_info.delay=PostscriptGeometry(argv[i]);
+                }
+              break;
+            }
           if (strncmp("density",option+1,3) == 0)
             {
               image_info.density=(char *) NULL;
@@ -513,13 +529,13 @@ int main(int argc,char **argv)
             }
           if (strncmp("dispose",option+1,5) == 0)
             {
-              image_info.dispose=0;
+              image_info.dispose=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
                     Error("Missing method on -dispose",(char *) NULL);
-                  image_info.dispose=atoi(argv[i]);
+                  image_info.dispose=PostscriptGeometry(argv[i]);
                 }
               break;
             }
@@ -639,7 +655,7 @@ int main(int argc,char **argv)
         {
           if (strncmp("help",option+1,2) == 0)
             {
-              Usage();
+              Usage(client_name);
               break;
             }
           Error("Unrecognized option",option);
@@ -673,6 +689,8 @@ int main(int argc,char **argv)
                     image_info.interlace=LineInterlace;
                   if (Latin1Compare("plane",option) == 0)
                     image_info.interlace=PlaneInterlace;
+                  if (Latin1Compare("partition",option) == 0)
+                    image_info.interlace=PartitionInterlace;
                   if (image_info.interlace == UndefinedInterlace)
                     Error("Invalid interlace type on -interlace",option);
                 }
@@ -693,6 +711,28 @@ int main(int argc,char **argv)
                 }
               break;
             }
+          if (strncmp("linewidth",option+1,2) == 0)
+            {
+              if (*option == '-')
+                {
+                  i++;
+                  if ((i == argc) || !sscanf(argv[i],"%d",&x))
+                    Error("Missing size on -linewidth",(char *) NULL);
+                }
+              break;
+            }
+          if (strncmp("loop",option+1,2) == 0)
+            {
+              image_info.iterations=(char *) NULL;
+              if (*option == '-')
+                {
+                  i++;
+                  if ((i == argc) || !sscanf(argv[i],"%d",&x))
+                    Error("Missing iterations on -loop",(char *) NULL);
+                  image_info.iterations=PostscriptGeometry(argv[i]);
+                }
+              break;
+            }
           Error("Unrecognized option",option);
           break;
         }
@@ -700,15 +740,12 @@ int main(int argc,char **argv)
         {
           if (strncmp("map",option+1,3) == 0)
             {
+              global_colormap=(*option == '+');
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     Error("Missing file name on -map",(char *) NULL);
-                  (void) strcpy(image_info.filename,argv[i]);
-                  map_image=ReadImage(&image_info);
-                  if (map_image == (Image *) NULL)
-                    Error("Unable to read map image",argv[i]);
                 }
               break;
             }
@@ -735,12 +772,6 @@ int main(int argc,char **argv)
           if (strncmp("monochrome",option+1,4) == 0)
             {
               image_info.monochrome=(*option == '-');
-              if (image_info.monochrome)
-                {
-                  number_colors=2;
-                  tree_depth=8;
-                  colorspace=GRAYColorspace;
-                }
               break;
             }
           Error("Unrecognized option",option);
@@ -888,7 +919,7 @@ int main(int argc,char **argv)
               if (*option == '-')
                 {
                   i++;
-                  if ((i == argc) || !sscanf(argv[i],"%f",(float *) &x))
+                  if ((i == argc) || !IsGeometry(argv[i]))
                     Error("Missing degrees on -rotate",(char *) NULL);
                 }
               break;
@@ -1016,6 +1047,16 @@ int main(int argc,char **argv)
                 }
               break;
             }
+          if (strncmp("threshold",option+1,2) == 0)
+            {
+              if (*option == '-')
+                {
+                  i++;
+                  if ((i == argc) || !sscanf(argv[i],"%d",&x))
+                    Error("Missing value on -threshold",(char *) NULL);
+                }
+              break;
+            }
           if (strncmp("transparent",option+1,3) == 0)
             {
               if (*option == '-')
@@ -1028,13 +1069,11 @@ int main(int argc,char **argv)
             }
           if (strncmp("treedepth",option+1,3) == 0)
             {
-              tree_depth=0;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
                     Error("Missing depth on -treedepth",(char *) NULL);
-                  tree_depth=atoi(argv[i]);
                 }
               break;
             }
@@ -1066,7 +1105,7 @@ int main(int argc,char **argv)
         }
         case '?':
         {
-          Usage();
+          Usage(client_name);
           break;
         }
         default:
@@ -1077,9 +1116,6 @@ int main(int argc,char **argv)
       }
     else
       {
-        Image
-          *image;
-
         /*
           Option is a file name: begin by reading image from specified file.
         */
@@ -1113,75 +1149,35 @@ int main(int argc,char **argv)
                 (void) strcpy(image->filename,image_info.filename);
               }
           }
-         /*
+        /*
           Transmogrify image as defined by the image processing options.
         */
         MogrifyImages(&image_info,i,argv,&image);
-        if (map_image != (Image *) NULL)
-          MapImage(image,map_image,False);
-        if (number_colors == 0)
-          if (format != (char *) NULL)
-            {
-              AppendImageFormat(format,image->filename);
-              (void) WriteImage(&image_info,image);
-            }
-          else
-            {
-              /*
-                Write image to temporary file.
-              */
-              (void) strcpy(filename,image->filename);
-              LocaleFilename(image->filename);
-              status=WriteImage(&image_info,image);
-              if (status == True)
-                (void) rename(image->filename,filename);
-            }
-        if (image_number == maximum_images)
-          {
-            /*
-              Increase size of images array.
-            */
-            maximum_images<<=1;
-            images=(Image **)
-              realloc((char *) images,maximum_images*sizeof(Image *));
-            if (images == (Image **) NULL)
-              Error("Unable to mogrify images","Memory allocation failed");
-          }
-        images[image_number++]=image;
-        if (number_colors == 0)
-          DestroyImage(image);
-      }
-    }
-  if (image_number == 0)
-    Error("Missing an image file name",(char *) NULL);
-  if (number_colors != 0)
-    {
-      /*
-        Global colormap option;  reduce colors then write image.
-      */
-      QuantizeImages(images,image_number,number_colors,tree_depth,
-        image_info.dither,colorspace);
-      for (i=0; i < image_number; i++)
-      {
+        if (global_colormap)
+          MapImages(image,(Image *) NULL,image_info.dither);
+        /*
+          Write transmogified image to disk.
+        */
         if (format != (char *) NULL)
           {
-            AppendImageFormat(format,images[i]->filename);
-            (void) WriteImage(&image_info,images[i]);
+            AppendImageFormat(format,image->filename);
+            (void) WriteImage(&image_info,image);
+            DestroyImage(image);
+            continue;
           }
-        else
+        (void) strcpy(filename,image->filename);
+        LocaleFilename(image->filename);
+        status=WriteImage(&image_info,image);
+        if (status == True)
           {
-            /*
-              Write image to temporary file.
-            */
-            (void) strcpy(filename,images[i]->filename);
-            LocaleFilename(images[i]->filename);
-            status=WriteImage(&image_info,images[i]);
-            if (status == True)
-              (void) rename(images[i]->filename,filename);
+            (void) remove(filename);
+            (void) rename(image->filename,filename);
           }
-        DestroyImage(images[i]);
+        DestroyImage(image);
       }
     }
+  if (image == (Image *) NULL)
+    Error("Missing an image file name",(char *) NULL);
   exit(0);
   return(False);
 }
