@@ -19,7 +19,7 @@
 %                              July 1992                                      %
 %                                                                             %
 %                                                                             %
-%  Copyright 1996 E. I. du Pont de Nemours and Company                        %
+%  Copyright 1997 E. I. du Pont de Nemours and Company                        %
 %                                                                             %
 %  Permission to use, copy, modify, distribute, and sell this software and    %
 %  its documentation for any purpose is hereby granted without fee,           %
@@ -37,7 +37,7 @@
 %  and fitness, in no event shall E. I. du Pont de Nemours and Company be     %
 %  liable for any special, indirect or consequential damages or any           %
 %  damages whatsoever resulting from loss of use, data or profits, whether    %
-%  in an action of contract, negligence or other tortuous action, arising     %
+%  in an action of contract, negligence or other tortious action, arising     %
 %  out of or in connection with the use or performance of this software.      %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -83,36 +83,39 @@
 %  Usage: montage [options ...] file [ [options ...] file ...] file
 %
 %  Where options include:
-%    -adjoin            join images into a single multi-image file
-%    -colors value      preferred number of colors in the image
-%    -colorspace type   GRAY, OHTA, RGB, XYZ, YCbCr, YIQ, YPbPr, or YUV
-%    -comment string    annotate image with comment
-%    -compose operator  composite operator
-%    -compress type     RunlengthEncoded or Zip
-%    -crop geometry     preferred size and location of the cropped image
-%    -density geometry  vertical and horizontal density of the image
-%    -display server    query fonts from this X server
-%    -dispose method    GIF disposal method
-%    -dither            apply Floyd/Steinberg error diffusion to image
-%    -frame             surround image with an ornamental border
-%    -gamma value       level of gamma correction
-%    -geometry geometry preferred tile and border sizes
-%    -gravity direction which direction to gravitate towards
-%    -interlace type    NONE, LINE, or PLANE
-%    -label name        assign a label to an image
-%    -monochrome        transform image to black and white
-%    -page geometry     size and location of the Postscript page
-%    -pointsize value   pointsize of Postscript font
-%    -quality value     JPEG quality setting
-%    -rotate degrees    apply Paeth rotation to the image
-%    -scene value       image scene number
-%    -shadow            add a shadow beneath a tile to simulate depth
-%    -size geometry     width and height of image
-%    -texture filename  name of texture to tile onto the image background
-%    -tile geometry     number of tiles per row and column
-%    -transparent color make this color transparent within the image
-%    -treedepth value   depth of the color classification tree
-%    -verbose           print detailed information about the image
+%    -adjoin             join images into a single multi-image file
+%    -colors value       preferred number of colors in the image
+%    -colorspace type    GRAY, OHTA, RGB, XYZ, YCbCr, YIQ, YPbPr, or YUV
+%    -comment string     annotate image with comment
+%    -compose operator   composite operator
+%    -compress type      RunlengthEncoded or Zip
+%    -crop geometry      preferred size and location of the cropped image
+%    -density geometry   vertical and horizontal density of the image
+%    -display server     query fonts from this X server
+%    -dispose method     GIF disposal method
+%    -dither             apply Floyd/Steinberg error diffusion to image
+%    -draw string        annotate the image with a graphic primitive
+%    -frame geometry     surround image with an ornamental border
+%    -gamma value        level of gamma correction
+%    -geometry geometry  preferred tile and border sizes
+%    -gravity direction  which direction to gravitate towards
+%    -interlace type     None, Line, Plane, or Partition
+%    -label name         assign a label to an image
+%    -matte              store matte channel if the image has one
+%    -mode type          Frame, Unframe, or Concatenate
+%    -monochrome         transform image to black and white
+%    -page geometry      size and location of the Postscript page
+%    -pointsize value    pointsize of Postscript font
+%    -quality value      JPEG quality setting
+%    -rotate degrees     apply Paeth rotation to the image
+%    -scene value        image scene number
+%    -shadow             add a shadow beneath a tile to simulate depth
+%    -size geometry      width and height of image
+%    -texture filename   name of texture to tile onto the image background
+%    -tile geometry      number of tiles per row and column
+%    -transparent color  make this color transparent within the image
+%    -treedepth value    depth of the color classification tree
+%    -verbose            print detailed information about the image
 %
 %  In addition to those listed above, you can specify these standard X
 %  resources as command line options:  -background, -bordercolor -borderwidth,
@@ -136,12 +139,6 @@
 #include "magick.h"
 #include "version.h"
 
-/*
-  Function prototypes.
-*/
-static void
-  Usage(void);
-
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -157,11 +154,14 @@ static void
 %
 %  The format of the Usage routine is:
 %
-%      Usage()
+%      Usage(client_name)
+%
+%    o client_name: a character string representing the name of the client
+%      program.
 %
 %
 */
-static void Usage(void)
+static void Usage(const char *client_name)
 {
   char
     **p;
@@ -180,12 +180,15 @@ static void Usage(void)
       "-display server     query font from this X server",
       "-dispose method     GIF disposal method",
       "-dither             apply Floyd/Steinberg error diffusion to image",
-      "-frame              surround image with an ornamental border",
+      "-draw string        annotate the image with a graphic primitive",
+      "-frame geometry     surround image with an ornamental border",
       "-gamma value        level of gamma correction",
       "-geometry geometry  preferred tile and border sizes",
       "-gravity direction  which direction to gravitate towards",
-      "-interlace type     NONE, LINE, or PLANE",
+      "-interlace type     None, Line, Plane, or Partition",
       "-label name         assign a label to an image",
+      "-matte              store matte channel if the image has one",
+      "-mode type          Frame, Unframe, or Concatenate",
       "-monochrome         transform image to black and white",
       "-page geometry      size and location of the Postscript page",
       "-pointsize value    pointsize of Postscript font",
@@ -247,6 +250,7 @@ static void Usage(void)
 int main(int argc,char **argv)
 {
   char
+    *client_name,
     *commands[3],
     *option,
     *server_name,
@@ -261,17 +265,20 @@ int main(int argc,char **argv)
     *next_image;
 
   ImageInfo
-    image_info;
+    image_info,
+    local_info;
 
   int
     i,
     x;
 
+  MontageMode
+    mode;
+
   unsigned int
     first_scene,
     last_scene,
-    scene,
-    verbose;
+    scene;
 
   XMontageInfo
     montage_info;
@@ -288,7 +295,7 @@ int main(int argc,char **argv)
   ReadCommandlLine(argc,&argv);
   client_name=ClientName(*argv);
   if (argc < 3)
-    Usage();
+    Usage(client_name);
   /*
     Set defaults.
   */
@@ -297,11 +304,11 @@ int main(int argc,char **argv)
   GetImageInfo(&image_info);
   image=(Image *) NULL;
   last_scene=0;
+  mode=FrameMode;
   XGetMontageInfo(&montage_info);
   resource_database=(XrmDatabase) NULL;
   resource_info.background_color=DefaultTileBackground;
-  resource_info.border_color=DefaultTileBorder;
-  resource_info.border_width=atoi(DefaultTileBorderwidth);;
+  resource_info.border_width=atoi(DefaultTileBorderwidth);
   resource_info.colorspace=RGBColorspace;
   resource_info.dither=False;
   resource_info.foreground_color=DefaultTileForeground;
@@ -344,7 +351,7 @@ int main(int argc,char **argv)
         break;
       }
     if (strncmp("help",option+1,2) == 0)
-      Usage();
+      Usage(client_name);
   }
   /*
     Open X server connection.
@@ -377,7 +384,7 @@ int main(int argc,char **argv)
         XGetResourceClass(resource_database,client_name,"frame","True");
       montage_info.frame=IsTrue(resource_value);
       resource_value=
-        XGetResourceClass(resource_database,client_name,"interlace","none");
+        XGetResourceClass(resource_database,client_name,"interlace","plane");
       image_info.interlace=UndefinedInterlace;
       if (Latin1Compare("none",resource_value) == 0)
         image_info.interlace=NoneInterlace;
@@ -385,6 +392,8 @@ int main(int argc,char **argv)
         image_info.interlace=LineInterlace;
       if (Latin1Compare("plane",resource_value) == 0)
         image_info.interlace=PlaneInterlace;
+      if (Latin1Compare("partition",resource_value) == 0)
+        image_info.interlace=PartitionInterlace;
       if (image_info.interlace == UndefinedInterlace)
         Warning("Unrecognized interlace type",resource_value);
       resource_info.image_geometry=XGetResourceInstance(resource_database,
@@ -629,13 +638,13 @@ int main(int argc,char **argv)
             }
           if (strncmp("dispose",option+1,5) == 0)
             {
-              image_info.dispose=0;
+              image_info.dispose=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
                     Error("Missing method on -dispose",(char *) NULL);
-                  image_info.dispose=atoi(argv[i]);
+                  image_info.dispose=PostscriptGeometry(argv[i]);
                 }
               break;
             }
@@ -644,6 +653,16 @@ int main(int argc,char **argv)
               resource_info.dither=(*option == '-');
               break;
             }
+          if (strncmp("draw",option+1,2) == 0)
+            {
+              if (*option == '-')
+                {
+                  i++;
+                  if (i == argc)
+                    Error("Missing primitive on -draw",(char *) NULL);
+                }
+              break;
+            }
           Error("Unrecognized option",option);
           break;
         }
@@ -676,15 +695,14 @@ int main(int argc,char **argv)
             }
           if (strncmp("frame",option+1,2) == 0)
             {
-              montage_info.frame=(*option == '-');
+              montage_info.frame=False;
+              resource_info.border_width=0;
               if (*option == '-')
                 {
                   i++;
-                  if (i == argc)
+                  if ((i == argc) || !IsGeometry(argv[i]))
                     Error("Missing geometry on -frame",(char *) NULL);
                 }
-              if (!montage_info.frame)
-                resource_info.border_width=0;
               break;
             }
           Error("Unrecognized option",option);
@@ -758,7 +776,7 @@ int main(int argc,char **argv)
         {
           if (strncmp("help",option+1,2) == 0)
             {
-              Usage();
+              Usage(client_name);
               break;
             }
           Error("Unrecognized option",option);
@@ -782,6 +800,8 @@ int main(int argc,char **argv)
                     image_info.interlace=LineInterlace;
                   if (Latin1Compare("plane",option) == 0)
                     image_info.interlace=PlaneInterlace;
+                  if (Latin1Compare("partition",option) == 0)
+                    image_info.interlace=PartitionInterlace;
                   if (image_info.interlace == UndefinedInterlace)
                     Error("Invalid interlace type on -interlace",option);
                 }
@@ -807,6 +827,8 @@ int main(int argc,char **argv)
         }
         case 'm':
         {
+          if (strcmp("matte",option+1) == 0)
+            break;
           if (strncmp("mattecolor",option+1,6) == 0)
             {
               resource_info.matte_color=(char *) NULL;
@@ -819,7 +841,45 @@ int main(int argc,char **argv)
                 }
               break;
             }
-          if (strncmp("monochrome",option+1,2) == 0)
+          if (strncmp("mode",option+1,4) == 0)
+            {
+              if (*option == '-')
+                {
+                  i++;
+                  if (i == argc)
+                    Error("Missing type on -mode",(char *) NULL);
+                  option=argv[i];
+                  mode=UndefinedMode;
+                  if (Latin1Compare("frame",option) == 0)
+                    {
+                      mode=FrameMode;
+                      montage_info.frame=True;
+                      montage_info.shadow=True;
+                      break;
+                    }
+                  if (Latin1Compare("unframe",option) == 0)
+                    {
+                      mode=UnframeMode;
+                      montage_info.frame=False;
+                      montage_info.shadow=False;
+                      resource_info.border_width=0;
+                      break;
+                    }
+                  if (Latin1Compare("concatenate",option) == 0)
+                    {
+                      mode=ConcatenateMode;
+                      montage_info.frame=False;
+                      montage_info.shadow=False;
+                      resource_info.image_geometry="+0+0";
+                      resource_info.border_width=0;
+                      break;
+                    }
+                  if (mode == UndefinedMode)
+                    Error("Invalid mode type on -mode",option);
+                }
+              break;
+            }
+          if (strncmp("monochrome",option+1,3) == 0)
             {
               resource_info.monochrome=(*option == '-');
               if (resource_info.monochrome)
@@ -886,7 +946,7 @@ int main(int argc,char **argv)
           if (*option == '-')
             {
               i++;
-              if ((i == argc) || !sscanf(argv[i],"%f",(float *) &x))
+              if ((i == argc) || !IsGeometry(argv[i]))
                 Error("Missing degrees on -rotate",(char *) NULL);
             }
           break;
@@ -1010,7 +1070,7 @@ int main(int argc,char **argv)
         }
         case '?':
         {
-          Usage();
+          Usage(client_name);
           break;
         }
         default:
@@ -1051,23 +1111,22 @@ int main(int argc,char **argv)
             break;
           else
             continue;
-        MogrifyImages(&image_info,i,argv,&next_image);
-        verbose=image_info.verbose;
-        image_info.verbose=False;
+        local_info=image_info;
+        local_info.verbose=False;
         commands[0]=client_name;
-        if (!next_image->label)
+        if ((mode != ConcatenateMode) && !next_image->label)
           {
             /*
               Default tile label is the filename.
             */
             commands[1]="-label";
             commands[2]=DefaultTileLabel;
-            MogrifyImages(&image_info,3,commands,&next_image);
+            MogrifyImages(&local_info,3,commands,&next_image);
           }
         commands[1]="-geometry";
         commands[2]=resource_info.image_geometry;
-        MogrifyImages(&image_info,3,commands,&next_image);
-        image_info.verbose=verbose;
+        MogrifyImages(&local_info,3,commands,&next_image);
+        MogrifyImages(&image_info,i,argv,&next_image);
         if (image == (Image *) NULL)
           image=next_image;
         else
