@@ -136,23 +136,23 @@ static long
   Forward declarations.
 */
 static char
-  *Atom _Declare((int *)),
-  *Branch _Declare((int *)),
-  *NextToken _Declare((register char *)),
-  *Node _Declare((int)),
-  *Piece _Declare((int *)),
-  *Regular _Declare((int,int *));
+  *Atom(int *),
+  *Branch(int *),
+  *NextToken(register char *),
+  *Node(int),
+  *Piece(int *),
+  *Regular(int,int *);
 
 static int
-  Match _Declare((char *)),
-  Repeat _Declare((char *)),
-  Try _Declare((RegularExpression *,char *));
+  Match(char *),
+  Repeat(char *),
+  Try(RegularExpression *,char *);
 
 static void
-  EmitCode _Declare((int)),
-  Insert _Declare((int,char *)),
-  OpTail _Declare((char *,char *)),
-  Tail _Declare((char *,char *));
+  EmitCode(int),
+  Insert(int,char *),
+  OpTail(char *,char *),
+  Tail(char *,char *);
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -167,9 +167,7 @@ static void
 %
 %
 */
-static char *Atom(flagp)
-int
-  *flagp;
+static char *Atom(int *flagp)
 {
   int
     flags;
@@ -320,9 +318,7 @@ int
 %
 %
 */
-static char *Branch(flagp)
-int
-  *flagp;
+static char *Branch(int *flagp)
 {
   int
     flags;
@@ -365,9 +361,7 @@ int
 %
 %
 */
-static void EmitCode(opcode)
-int
-  opcode;
+static void EmitCode(int opcode)
 {
   if (code != &start_code)
     *code++=(char) opcode;
@@ -389,12 +383,7 @@ int
 %  Function Insert inserts an operator in front of an already-emitted operand.
 %
 */
-static void Insert(opcode,operand)
-int
-  opcode;
-
-char
-  *operand;
+static void Insert(int opcode,char *operand)
 {
   register char
     *p,
@@ -430,9 +419,7 @@ char
 %
 %
 */
-static int Match(regular_expression)
-char
-  *regular_expression;
+static int Match(char *regular_expression)
 {
   char
     *next_token;
@@ -653,9 +640,7 @@ char
 %
 %
 */
-static char *NextToken(p)
-register char
-  *p;
+static char *NextToken(register char *p)
 {
   register int
     offset;
@@ -684,9 +669,7 @@ register char
 %
 %
 */
-static char *Node(opcode)
-int
-  opcode;
+static char *Node(int opcode)
 {
   register char
     *ptr,
@@ -719,12 +702,7 @@ int
 %
 %
 */
-static void OpTail(p,value)
-char
-  *p;
-
-char
-  *value;
+static void OpTail(char *p,char *value)
 {
   /*
     "Operandless" and "op != MatchThisOrNext" are synonymous in practice.
@@ -747,9 +725,7 @@ char
 %
 %
 */
-static char *Piece(flagp)
-int
-  *flagp;
+static char *Piece(int *flagp)
 {
   int
     flags;
@@ -831,12 +807,7 @@ int
 %
 %
 */
-static char *Regular(parenthesized,flagp)
-int
-  parenthesized;
-
-int
-  *flagp;
+static char *Regular(int parenthesized,int *flagp)
 {
   int
     flags;
@@ -927,9 +898,7 @@ int
 %
 %
 */
-static int Repeat(p)
-char
-  *p;
+static int Repeat(char *p)
 {
   register char
     *operand,
@@ -999,12 +968,7 @@ char
 %
 %
 */
-static void Tail(p,val)
-char
-  *p;
-
-char
-  *val;
+static void Tail(char *p,char *val)
 {
   register char
     *scan,
@@ -1047,12 +1011,7 @@ char
 %
 %
 */
-static int Try(regular_expression,pattern)
-RegularExpression
-  *regular_expression;
-
-char
-  *pattern;
+static int Try(RegularExpression *regular_expression,char *pattern)
 {
   register char
     **ep,
@@ -1099,9 +1058,7 @@ char
 %
 %
 */
-RegularExpression *CompileRegularExpression(regular_expression)
-char
-  *regular_expression;
+RegularExpression *CompileRegularExpression(char *regular_expression)
 {
   int
     flags;
@@ -1200,12 +1157,8 @@ char
 %
 %
 */
-int ExecuteRegularExpression(regular_expression,pattern)
-register RegularExpression
-  *regular_expression;
-
-register char
-  *pattern;
+int ExecuteRegularExpression(register RegularExpression *regular_expression,
+  register char *pattern)
 {
   register char
     *s;
